package com.jz.common.bean;

import java.lang.reflect.Field;

import org.apache.commons.lang3.StringUtils;

import com.jz.common.annotation.FieldAssist;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

/**分页信息封装
 * @author ybz
 *
 */
@ApiModel(value = "分页信息", description = "分页信息封装")
public class BasePageBean {
	
	@ApiModelProperty(value = "当前页码")
	private int pageNum = 1;
	
	@ApiModelProperty(value = "每页显示记录数")
	private int pageSize = 10;
	
	@ApiModelProperty(value = "排序字段")
	private String orderColumn;
	
	@ApiModelProperty(value = "排序方式(asc：升序；desc：降序)")
	private String orderType;
	
	public int getPageNum() {
		return pageNum;
	}

	public void setPageNum(int pageNum) {
		this.pageNum = pageNum;
	}

	public int getPageSize() {
		return pageSize;
	}

	public void setPageSize(int pageSize) {
		this.pageSize = pageSize;
	}

	public String getOrderColumn() {
		return orderColumn;
	}

	public void setOrderColumn(String orderColumn) {
		this.orderColumn = orderColumn;
	}

	public String getOrderType() {
		return orderType;
	}

	public void setOrderType(String orderType) {
		this.orderType = orderType;
	}
	
	//获取排序方式字符串
	public String takeOrderByStr(Class<?> cls) {
		if (StringUtils.isEmpty(orderColumn) || StringUtils.isEmpty(orderType)) {
			return null;
		}
		
		try {
			Field field = cls.getDeclaredField(orderColumn);
			boolean fieldHasAnno = field.isAnnotationPresent(FieldAssist.class);
			if (fieldHasAnno) {
				FieldAssist fieldAssist = field.getAnnotation(FieldAssist.class);
				String orderBy = fieldAssist.orderBy();
				this.orderColumn = orderBy;
			}
		} catch (Exception e) {
			throw new RuntimeException("该排序字段没有找到对应的排序字段");
		}
		
		return orderColumn+ " " + orderType;
	}

}
