package com.jz.common.utils.realTime;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.*;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.util.MimeType;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.client.RestTemplate;

import java.nio.charset.Charset;
import java.util.Map;

/**
 * 使用RestTemplate发送HTTP请求
 *
 * @return
 * @author Bellamy
 * @since 2021-01-08
 */
public class RestClient {
	
	public static final Logger LOGGER = LoggerFactory.getLogger(RestClient.class);
	
	/**
	 * 
	 * @param connetionTimeOut
	 * @param socketTimeOut
	 * @return
	 */
	public static RestTemplate getRestTemplate() {
		HttpComponentsClientHttpRequestFactory requestFactory = new HttpComponentsClientHttpRequestFactory();
	    requestFactory.setConnectTimeout(120 * 1000); 
	    requestFactory.setReadTimeout(480 * 1000);
	    RestTemplate restTemplate = new RestTemplate(requestFactory);
	    return restTemplate;
	}
	
	
	
	/** 
	 * 使用RestTemplate发送HTTP POST请求 
	 * @param url 请求地址 
	 * @param jsonStr 请求参数 
	 * @param connectionTimeout 连接超时时间 
	 * @param socketTimeout socket超时时间 
	 * @return Map 返回报文 
	 */  
	@SuppressWarnings({ "unchecked" })
	public static Map<String, Object> post(String url,String jsonStr){
		LOGGER.info("===================post request Start=======================");
		LOGGER.info("url:" + url);
		LOGGER.info("json:" + jsonStr);
	    HttpHeaders headers = new HttpHeaders();
	    headers.set("Content-Type", "application/json;charset=UTF-8");//解决请求乱码问题 
	    Map<String, Object> resutMap = null ;
	    try {  
	    	RestTemplate restTemplate = getRestTemplate();
	    	resutMap = restTemplate.postForObject(url, new HttpEntity<String>(jsonStr, headers), Map.class);
	    } catch (Exception e) { 
	    	e.printStackTrace();
	    	LOGGER.error("rest post 异常",e.getMessage(),e);
	    }
		LOGGER.info("===================post request end=======================");
	    return resutMap;  
	}
	
	
	/**
	 * 使用RestTemplate发送HTTP get请求
	 * @param url
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Map<String, Object> get(String url){
		RestTemplate restTemplate = getRestTemplate();
		Map<String,Object> map = null;
		try {
			map = restTemplate.getForObject(url, Map.class);
		} catch (Exception e) {
			LOGGER.error("rest get 异常",e.getMessage(),e);
		}
		return map;
	}
	
	/**
	 * 使用RestTemplate发送HTTP put请求
	 * @param url
	 * @param jsonStr
	 */
	public static Map<String, Object> put(String url,String jsonStr){
		Map<String, Object> map = null ;
		try {
			map = exchange(url, HttpMethod.PUT, jsonStr);
		} catch (Exception e) {
			LOGGER.error("rest put 异常",e.getMessage(),e);
		}
		return map ;
	}
	
	/**
	 * 使用RestTemplate发送HTTP delete 请求
	 * @param url
	 * @param jsonStr
	 */
	public static void delete(String url) {
		try {
			RestTemplate restTemplate = getRestTemplate();
			restTemplate.delete(url);
		} catch (Exception e) {
			LOGGER.error("rest delete 异常",e.getMessage(),e);
		}
		
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public static Map<String, Object> exchange(String url, HttpMethod method, String jsonStr) {
        // 请求头
        HttpHeaders headers = new HttpHeaders();
        MimeType mimeType = MimeTypeUtils.parseMimeType("application/json");
        MediaType mediaType = new MediaType(mimeType.getType(), mimeType.getSubtype(), Charset.forName("UTF-8"));
        // 请求体
        headers.setContentType(mediaType);
       
        // 发送请求
        HttpEntity<String> entity = new HttpEntity<>(jsonStr, headers);
        ResponseEntity<Map> resultEntity = getRestTemplate().exchange(url, method, entity, Map.class);
        return resultEntity.getBody();
    }
	
	
	public static void main(String[] args) {
		String url = "http://10.0.53.177:9883/connectors";
		post(url, jsonStr);
	}
	
	static String jsonStr = "{\r\n" + 
			"	\"name\": \"debezium-connector-avtiviti-process-center-1\",\r\n" + 
			"	\"config\": {\r\n" + 
			"		\"connector.class\": \"io.debezium.connector.mysql.MySqlConnector\",\r\n" + 
			"		\"database.history.producer.sasl.kerberos.service.name\": \"kafka\",\r\n" + 
			"		\"tasks.max\": \"3\",\r\n" + 
			"		\"database.history.kafka.topic\": \"avtiviti.process-center.databasehistory\",\r\n" + 
			"		\"column.blacklist\": \"\",\r\n" + 
			"		\"transforms\": \"unwrap\",\r\n" + 
			"		\"database.history.consumer.security.protocol\": \"SASL_PLAINTEXT\",\r\n" + 
			"		\"database.history.consumer.sasl.kerberos.service.name\": \"kafka\",\r\n" + 
			"		\"table.blacklist\": \"\",\r\n" + 
			"		\"transforms.unwrap.drop.tombstones\": \"true\",\r\n" + 
			"		\"transforms.unwrap.type\": \"io.debezium.transforms.UnwrapFromEnvelope\",\r\n" + 
			"		\"value.converter\": \"io.confluent.connect.avro.AvroConverter\",\r\n" + 
			"		\"database.whitelist\": \"process-center\",\r\n" + 
			"		\"key.converter\": \"io.confluent.connect.avro.AvroConverter\",\r\n" + 
			"		\"database.history.producer.sasl.mechanism\": \"GSSAPI\",\r\n" + 
			"		\"database.user\": \"root\",\r\n" + 
			"		\"database.server.id\": \"20\",\r\n" + 
			"		\"database.history.producer.security.protocol\": \"SASL_PLAINTEXT\",\r\n" + 
			"		\"database.history.kafka.bootstrap.servers\": \"SASL_PLAINTEXT://10.0.108.61:9092,SASL_PLAINTEXT://10.0.108.62:9092,SASL_PLAINTEXT://10.0.108.63:9092\",\r\n" + 
			"		\"database.server.name\": \"avtiviti\",\r\n" + 
			"		\"database.port\": \"3306\",\r\n" + 
			"		\"value.converter.schema.registry.url\": \"http://10.0.108.61:9881\",\r\n" + 
			"		\"database.hostname\": \"10.0.53.179\",\r\n" + 
			"		\"database.password\": \"jz@2018\",\r\n" + 
			"		\"database.history.consumer.sasl.mechanism\": \"GSSAPI\",\r\n" + 
			"		\"key.converter.schema.registry.url\": \"http://10.0.108.61:9881\"\r\n" + 
			"	}\r\n" + 
			"}";
	
}
