package com.jz.dmp.modules.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.agent.service.DmpDsAgentService;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.constant.CommConstant;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.constant.StatuConstant;
import com.jz.common.enums.FlowInstanceEnum;
import com.jz.common.enums.ModuleLogEnum;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.*;
import com.jz.common.utils.web.HttpClientUtils;
import com.jz.common.utils.web.XmlUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.azkaban.dao.ExecutionFlowsMapper;
import com.jz.dmp.modules.controller.DataIntegration.bean.flow.FlowPro;
import com.jz.dmp.modules.controller.bean.DmpDevelopTaskDto;
import com.jz.dmp.modules.controller.bean.DmpDevelopTaskRequest;
import com.jz.dmp.modules.controller.bean.MyDmpDevelopTaskConverter;
import com.jz.dmp.modules.controller.bean.MyDmpDevelopTaskHistoryConverter;
import com.jz.dmp.modules.controller.dataOperation.bean.*;
import com.jz.dmp.modules.controller.projconfig.bean.DmpProjectConfigInfoDto;
import com.jz.dmp.modules.dao.*;
import com.jz.dmp.modules.dao.projconfig.DmpProjectConfigInfoMapper;
import com.jz.dmp.modules.model.*;
import com.jz.dmp.modules.service.*;
import org.apache.hadoop.yarn.webapp.hamlet.Hamlet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.util.*;
import java.util.regex.Pattern;

/**
 * 任务开发(DmpDevelopTask)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-22 16:37:56
 */
@Service("dmpDevelopTaskService")
@Transactional
public class DmpDevelopTaskServiceImpl extends BaseService implements DmpDevelopTaskService {

    private static Logger logger = LoggerFactory.getLogger(OfflineSynchServiceImpl.class);

    @Autowired
    private DmpDevelopTaskDao dmpDevelopTaskDao;

    @Autowired
    private DmpProjectDao dmpProjectDao;

    @Autowired
    private DmpSyncingDatasourceTypeDao dmpSyncingDatasourceTypeDao;

    @Autowired
    private DmpDsAgentService dmpDsAgentServiceImp;

    @Autowired
    private DmpSyncingDatasourceService dmpSyncingDatasourceService;

    @Autowired
    private OfflineSynchService offlineSynchService;

    @Autowired
    private ExecutionFlowsMapper executionFlowsMapper;

    @Autowired
    private DmpModuleOperateLogService dmpModuleOperateLogService;

    @Autowired
    private DmpNavigationTreeDao dmpNavigationTreeDao;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private FlowService flowService;

    @Autowired
    private DmpDevelopTaskHistoryService dmpDevelopTaskHistoryService;

    @Autowired
    private DmpProjectConfigInfoMapper dmpProjectConfigInfoMapper;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private DmpRealtimeSyncInfoDao dmpRealtimeSyncInfoDao;

    /**
     * 添加保存dmp数据(包含校验数据)
     *
     * @Author Bellamy
     * @Date 2020/12/20
     */
    @Override
    public JsonResult submitSyncing(DmpDevelopTask task) throws Exception {
        String xmlContent;
        try {
            xmlContent = convert2SyncXmlContent(task);
            System.out.println(xmlContent);
        } catch (Exception e) {
            logger.error("封装离线同步xml内容出错：" + e.getMessage(), e);
            throw new RuntimeException(e.getMessage());
            //return JsonResult.error(ResultCode.INTERNAL_SERVER_ERROR,e.getMessage());
        }
        return submitSyncXml(task.getProjectId(), xmlContent);
    }

    /**
     * 提交离线同步到指定projectId项目下（上传xml配置文件）
     */
    private JsonResult submitSyncXml(long projectId, String xmlContent) {
        String _type_ = XmlUtils.getPropertyValue(xmlContent, "type");
        String _fileName_ = XmlUtils.getPropertyValue(xmlContent, "name");
        String _dbName_ = XmlUtils.getPropertyValue(xmlContent, "target_database");
        String _targetFtpConnection_ = XmlUtils.getPropertyValue(xmlContent, "target_ftp_connection");
        if (StringUtils.isNull(_dbName_)) { // 2ftp类型中无target_database标签
            if (_type_.toLowerCase().indexOf("2ftp") != -1) {
                _dbName_ = _targetFtpConnection_;
            } else {
                throw new RuntimeException("离线同步配置缺失目标数据源名称！");
            }
        }

        DmpAgentDatasourceInfo ds = new DmpAgentDatasourceInfo();
        DmpProjectSystemInfo info = dmpProjectDao.queryProjectSystemInfo(projectId);
        ds.setKerberosIsenable(info.getKerberosIsenable());
        ds.setKerberosJaasConf(info.getKerberosJaasConf());
        ds.setKerberosKrb5Conf(info.getKerberosKrb5Conf());
        ds.setKerberosFqdn(info.getKerberosFqdn());
        ds.setHdfsDefaultConf(""); // info.getHdfsDefaultConf());
        ds.setHdfsConfPath(""); // info.getHdfsConfPath());
        ds.setHdfsAuthPath(info.getKerberosKeytabConf()); // info.getHdfsAuthPath());
        ds.setHdfsUserName(info.getHdfsUserName());
        ds.setHdfsSyncingPath(info.getHdfsSyncingPath());
        ds.setDefaultFs(info.getHdfsHttpPath());

        ds.setTargetDbName(_dbName_);
        ds.setTargetFileName(_fileName_ + ".xml");
        ds.setProjectId(String.valueOf(projectId));
        DmpAgentResult rst = dmpDsAgentServiceImp.submitSycingXmlConfig(ds, xmlContent);

        if (!rst.getCode().val().equals("200")) {
            return new JsonResult(rst.getCode(), rst.getMessage());
        } else {
            rst.setResult(JsonMapper.fromJsonString(rst.getMessage(), Boolean.class));
            return new JsonResult(ResultCode.SUCCESS, rst);
        }
    }

    /*
     * 生成xml
     * */
    public String convert2SyncXmlContent(DmpDevelopTask body) throws Exception {
        logger.info("################################## 拼接xml 开始 ############################################");
        StringBuilder sb = new StringBuilder();
        long treeId = body.getTreeId();
        //通过树节点id 查询任务信息
        DmpDevelopTask task = dmpDevelopTaskDao.selectTaskInfoByParam(treeId);

        Map<String, Object> map = (Map<String, Object>) JsonMapper.fromJsonString(task.getScript(), Map.class);

        Map<String, Object> scriptMap = (Map<String, Object>) map.get("scripts");
        Object content = scriptMap.get("content");
        if (null != content && StringUtils.hasLength(content.toString())) {
            // 脚本模式-XML内容
            sb.append(content);
        } else {
            // 脚本模式-json内容，解析JSON串并组装XML内容
            Map<String, Object> settingMap = (Map<String, Object>) scriptMap.get("setting");
            String _extract = (String) settingMap.get("extract");
            String _extractExpression = (String) settingMap.get("extractExpression"); //增量表达式
            String _targetBucketCounts = (String) settingMap.get("targetBucketCounts");
            String _errorLimitRecord = (String) settingMap.get("errorLimitRecord");//错误记录数超过
            String _executorMemory = (String) settingMap.get("executorMemory"); //分配任务内存
            String _executorCores = (String) settingMap.get("executorCores"); //单executor的cpu数
            String _totalExecutorCores = (String) settingMap.get("totalExecutorCores"); //总executor的cpu数
            String _ftColumn = (String) settingMap.get("ftColumn");//分桶字段
            String _ftCount = (String) settingMap.get("ftCount");//分桶个数
            String _separateMax = (String) settingMap.get("separateMax");//分桶字段最大值
            String _separateMin = (String) settingMap.get("separateMin");//分桶字段最小值
            String _syncRate = (String) settingMap.get("syncRate");//同步速率
            String _maxConcurrency = (String) settingMap.get("maxConcurrency");//最大并发数
            String _preImportStatement = (String) settingMap.get("preImportStatement");//导入前语句
            String _postImportStatement = (String) settingMap.get("postImportStatement");//导入后语句

            //******源数据******
            Map<String, Object> readerMap = (Map<String, Object>) scriptMap.get("reader");
            String _dbConnection = (String) readerMap.get("dbConnection"); //源数据库名称
            String _fileType = (String) readerMap.get("fileType"); //文件类型
            String _sourceHdfsPath = (String) readerMap.get("sourceHdfsPath"); //HDFS存储目录
            String _sourceHdfsFile = (String) readerMap.get("sourceHdfsFile");
            String _sourceFtpDir = (String) readerMap.get("sourceFtpDir"); //文件所在目录
            String _sourceFtpFile = (String) readerMap.get("sourceFtpFile"); //文件名
            String _sourceSkipFtpFile = (String) readerMap.get("sourceSkipFtpFile"); //没有数据文件是否跳过
            String _sourceCsvDelimiter = (String) readerMap.get("sourceCsvDelimiter"); //分隔符
            String _sourceCsvHeader = (String) readerMap.get("sourceCsvHeader"); //是否含有表头
            String _sourceCsvCharset = (String) readerMap.get("sourceCsvCharset"); //字符集编码
            String _sourceCsvQuote = (String) readerMap.get("sourceCsvQuote");
            String _sourceFtpLoadDate = (String) readerMap.get("sourceFtpLoadDate");  //加载数据日期
            String _registerTableName = (String) readerMap.get("registerTableName");//源数据库表名称
            String registerTableName_ = (String) readerMap.get("registerTableName");
            String _dayByDay = (String) readerMap.get("dayByDay"); //dayByDay
            List<Map<String, Object>> _readerColumns = (List<Map<String, Object>>) readerMap.get("column"); //源数据库表字段

            //******目标数据******
            Map<String, Object> writerMap = (Map<String, Object>) scriptMap.get("writer");
            String _targetDbConnection = (String) writerMap.get("targetDbConnection"); //目标数据库
            String _targetTable = (String) writerMap.get("targetTable"); //目标数据库表名称
            String _targetFtpDir = (String) writerMap.get("targetFtpDir");
            String _targetFtpFile = (String) writerMap.get("targetFtpFile");
            String _targetCsvDelimiter = (String) writerMap.get("targetCsvDelimiter");
            String _targetCsvHeader = (String) writerMap.get("targetCsvHeader");
            String _targetCsvCharset = (String) writerMap.get("targetCsvCharset");
            String _targetInsertMergeOverwrite = (String) writerMap.get("targetInsertMergeOverwrite");  //同步策略
            List<Map<String, Object>> _writerColumns = (List<Map<String, Object>>) writerMap.get("column");//目标数据库表字段

            Integer _projectId_ = body.getProjectId(); //项目id

            //根据项目id和数据源名称 查询数据信息
            DmpSyncingDatasource sDS = this.getDmpSyncingDatasource(_projectId_, _dbConnection);//源
            DmpSyncingDatasource tDS = this.getDmpSyncingDatasource(_projectId_, _targetDbConnection);//目标

            if (null == sDS || null == tDS || null == sDS.getDatasourceType() || null == tDS.getDatasourceType())
                throw new RuntimeException("同步数据来源或数据去向信息有误");
            //根据数据源类型ID 获取数据源类型
            DmpSyncingDatasourceType sDST = dmpSyncingDatasourceTypeDao.queryById(sDS.getDatasourceType());//源
            DmpSyncingDatasourceType tDST = dmpSyncingDatasourceTypeDao.queryById(tDS.getDatasourceType());//目标

            String _targetDbName_ = this.getTargetDBname(tDS);
            String _sourceDbName_ = this.getTargetDBname(sDS);
            String st = sDST.getDatasourceType();
            String tt = tDST.getDatasourceType();

            //获取文件类型
            String ft = this.getFileType(_fileType, _sourceHdfsFile, _sourceFtpFile);

            boolean isBigData2jdbc = ("HIVE".equals(st) || "KUDU".equals(st)) && "RDBMS".equals(tDST.getDatasourceCatecode());
            boolean isBigData2ftp = ("HIVE".equals(st) || "KUDU".equals(st)) && "FTP".equals(tt);

            if ("RDBMS".equals(sDST.getDatasourceCatecode()) && "HIVE".equals(tt)) {
                _registerTableName = sDS.getDatasourceName() + "_" + _registerTableName.substring(_registerTableName.indexOf(".") + 1);
            }

            String name_ = (StringUtils.hasLength(_targetDbName_) ? _targetDbName_ : tDS.getDatasourceName()) + "_" + _targetTable;
            String type_ = this.getEtlProcessType(sDST, tDST, _fileType);

            String[] sci = this.genSourceColumnsInfo(sDST, tDST, ft, _registerTableName, registerTableName_, _extract,
                    _extractExpression, _readerColumns, _sourceDbName_);
            String source_table_ = sci[0];
            String source_columns_ = sci[1];
            String source_sql_ = sci[2];

            String target_table_ = _targetTable;
            String[] tci = this.genTargetColumnsInfo(sDST, tDST, _targetTable, _targetBucketCounts, _writerColumns);
            String target_columns_ = tci[0];
            String target_primary_columns_ = tci[1];
            String target_partition_columns_ = tci[2];
            String target_table_creation_ = tci[3];

            String clean_rules_ = sci[3] + tci[4];
            String check_rules_ = sci[4] + tci[5];

            sb.append("<etlprocess>").append("\r\n");
            sb.append("<name>").append(name_).append("</name>").append("\r\n");
            sb.append("<type>").append(type_).append("</type>").append("\r\n");

            if ("RDBMS".equals(sDST.getDatasourceCatecode())) {
                sb.append("<source_table db_connection=\"").append(sDS.getDatasourceName()).append("\" register_table_name=\"")
                        .append(_registerTableName.substring(_registerTableName.indexOf(".") + 1)).append("\">").append(source_table_).append("</source_table>").append("\r\n");
            }
            if ("HIVE".equals(st) || "KUDU".equals(st)) {
                String sourceTableName = _registerTableName;
                if ("KUDU".equals(st)) {
                    sourceTableName = "impala::" + _sourceDbName_ + "." + _registerTableName;
                }
                sb.append("<source_table db_connection=\"").append(sDS.getDatasourceName()).append("\" register_table_name=\"")
                        .append(_registerTableName).append("\">").append(sourceTableName).append("</source_table>").append("\r\n");
            }

            if (isBigData2ftp) { // 前端考虑到源中可能包含字段信息，分开处理
                _sourceCsvDelimiter = _targetCsvDelimiter;
                _sourceCsvCharset = _targetCsvCharset;
                _sourceCsvHeader = _targetCsvHeader;
            }

            if ("csv".equals(ft) || "FTP".equals(st) || isBigData2ftp) {
                sb.append("<source_csv_delimiter>").append(_sourceCsvDelimiter).append("</source_csv_delimiter>").append("\r\n");
                sb.append("<source_csv_header>").append(_sourceCsvHeader).append("</source_csv_header>").append("\r\n");
                sb.append("<source_csv_charset>").append(_sourceCsvCharset).append("</source_csv_charset>").append("\r\n");
                sb.append("<source_csv_quote>").append(_sourceCsvQuote).append("</source_csv_quote>").append("\r\n");
            }

            if ("FTP".equals(st)) {
                sb.append("<source_ftp_connection>").append(_dbConnection).append("</source_ftp_connection>").append("\r\n");
                sb.append("<source_ftp_dir>").append(_sourceFtpDir).append("</source_ftp_dir>").append("\r\n");
                sb.append("<source_ftp_file>").append(_sourceFtpFile).append("</source_ftp_file>").append("\r\n");
                sb.append("<source_skip_ftp_file>").append(_sourceSkipFtpFile).append("</source_skip_ftp_file>").append("\r\n");
                sb.append("<source_ftp_load_date>").append(_sourceFtpLoadDate).append("</source_ftp_load_date>").append("\r\n");
            }

            if ("KUDU".equals(tt) || "HIVE".equals(tt)) {
                sb.append("<source_hdfs_path>").append(this.getSourceHdfsDir(_sourceHdfsPath, _sourceHdfsFile)).append("</source_hdfs_path>").append("\r\n");
                sb.append("<source_columns>").append(source_columns_).append("</source_columns>").append("\r\n");
            }
            sb.append("<source_sql>").append(source_sql_).append("</source_sql>").append("\r\n");

            if ("KUDU".equals(tt)) {
                target_table_ = "impala::" + _targetDbName_ + "." + _targetTable;
            }

            if (isBigData2ftp) {
                sb.append("<source_download_file>").append(_targetFtpFile).append("</source_download_file>").append("\r\n");
                sb.append("<source_database>").append(_sourceDbName_).append("</source_database>").append("\r\n");
                sb.append("<target_insert_merge_overwrite>").append("overwrite").append("</target_insert_merge_overwrite>").append("\r\n");
                sb.append("<target_ftp_connection>").append(_targetDbConnection).append("</target_ftp_connection>").append("\r\n");
                sb.append("<target_ftp_dir>").append(_targetFtpDir).append("</target_ftp_dir>").append("\r\n");
            } else {
                sb.append("<target_insert_merge_overwrite>").append(_targetInsertMergeOverwrite).append("</target_insert_merge_overwrite>").append("\r\n");
                sb.append("<target_db_connection>").append(_targetDbConnection).append("</target_db_connection>").append("\r\n");
                sb.append("<target_database>").append(_targetDbName_).append("</target_database>").append("\r\n");
                sb.append("<target_table>").append(target_table_).append("</target_table>").append("\r\n");
                sb.append("<target_primary_columns>").append(target_primary_columns_).append("</target_primary_columns>").append("\r\n");
                sb.append("<target_table_creation>").append(target_table_creation_).append("</target_table_creation>").append("\r\n");
                if (!isBigData2jdbc) {
                    sb.append("<target_columns>").append(target_columns_).append("</target_columns>").append("\r\n");
                    sb.append("<target_partition_columns>").append(target_partition_columns_).append("</target_partition_columns>").append("\r\n");
                }
            }

            sb.append("<ft_column>").append(_ftColumn).append("</ft_column>").append("\r\n");
            sb.append("<ft_count>").append(_ftCount).append("</ft_count>").append("\r\n");
            sb.append("<separate_max>").append(_separateMax).append("</separate_max>").append("\r\n");
            sb.append("<separate_min>").append(_separateMin).append("</separate_min>").append("\r\n");
            sb.append("<syncRate>").append(_syncRate).append("</syncRate>").append("\r\n");
            sb.append("<maxConcurrency>").append(_maxConcurrency).append("</maxConcurrency>").append("\r\n");
            sb.append("<preImportStatement>").append(_preImportStatement).append("</preImportStatement>").append("\r\n");
            sb.append("<postImportStatement>").append(_postImportStatement).append("</postImportStatement>").append("\r\n");
            sb.append("<errorLimitRecord>").append(_errorLimitRecord).append("</errorLimitRecord>").append("\r\n");

            sb.append("<executor_memory>").append(_executorMemory).append("</executor_memory>").append("\r\n");
            sb.append("<executor_cores>").append(_executorCores).append("</executor_cores>").append("\r\n");
            sb.append("<total_executor_cores>").append(_totalExecutorCores).append("</total_executor_cores>").append("\r\n");
            sb.append("<day_by_day>").append(_dayByDay).append("</day_by_day>").append("\r\n");
            sb.append("<clean_rules>").append(clean_rules_).append("</clean_rules>").append("\r\n");
            sb.append("<check_rules>").append(check_rules_).append("</check_rules>").append("\r\n");

            sb.append("</etlprocess>").append("\r\n");
        }
        return sb.toString();
    }

    /*
     *根据项目id和数据源名称 查询数据信息
     * */
    private DmpSyncingDatasource getDmpSyncingDatasource(Integer projectId, String dsName) throws Exception {
        DmpSyncingDatasource ds = new DmpSyncingDatasource();
        ds.setDataStatus("1");
        ds.setDatasourceName(dsName);
        ds.setProjectId(projectId);
        List<DmpSyncingDatasource> dsList = dmpSyncingDatasourceService.findListByParams(ds);
        if (dsList != null && dsList.size() > 0) {
            return dsList.get(0);
        } else {
            return null;
        }
    }

    /*
     * 获取文件类型
     * */
    private String getFileType(String ft, String _sourceHdfsFile, String _sourceFtpFile) {
        if (StringUtils.isBlank(ft)) {
            if (StringUtils.isNotBlank(_sourceHdfsFile)) {
                int idx = _sourceHdfsFile.lastIndexOf(".");
                if (idx > -1) {
                    String nft = _sourceHdfsFile.substring(idx + 1, _sourceHdfsFile.length());
                    if (StringUtils.isNotBlank(nft)) {
                        if ("csv".equalsIgnoreCase(nft)) {
                            ft = "csv";
                        } else if ("json".equalsIgnoreCase(nft)) {
                            ft = "json";
                        } else if ("txt".equalsIgnoreCase(nft)) {
                            ft = "csv";
                        } else {
                            ft = "csv";
                        }
                    }
                }
            }
            if (StringUtils.isNotBlank(_sourceFtpFile)) {
                int idx = _sourceFtpFile.lastIndexOf(".");
                if (idx > -1) {
                    String nft = _sourceFtpFile.substring(idx + 1, _sourceFtpFile.length());
                    if (StringUtils.isNotBlank(nft)) {
                        if ("csv".equalsIgnoreCase(nft)) {
                            ft = "csv";
                        } else if ("json".equalsIgnoreCase(nft)) {
                            ft = "json";
                        } else if ("txt".equalsIgnoreCase(nft)) {
                            ft = "csv";
                        } else {
                            ft = "csv";
                        }
                    }
                }
            }
        }
        return ft;
    }

    private String getTargetDBname(DmpSyncingDatasource ds) {
        String url = ds.getJdbcUrl();
        if (StringUtils.isNotBlank(url)) {
            if (ds.getDatasourceType() == 2) {//sqlserver
                String[] arr1 = url.split("=");
                String[] arr2 = arr1[1].split(";");
                System.out.println(arr2[0]);
                return arr2[0];
            } else {
                String[] arr1 = url.split("/");
                String ls = arr1[arr1.length - 1];
                String[] arr2 = ls.split("\\?");
                return arr2[0];
            }

        } else {
            if (ds.getDatasourceType().equals("11")) {//ftp
                return ds.getDbName();
            }
            return "";
        }
    }

    private String getEtlProcessType(DmpSyncingDatasourceType s, DmpSyncingDatasourceType t, String ft) {
        String sT = s.getDatasourceType().toLowerCase();
        String tT = t.getDatasourceType().toLowerCase();
        String _ft = "";
        if (StringUtils.isNotBlank(ft)) {
            _ft = ft;
        }
        if (s.getDatasourceCatecode().equals("RDBMS")) {
            sT = "jdbc";
        }
        return sT + _ft + "2" + tT;
    }

    private String getTypeSize(String type) {
        if (StringUtils.isNull(type))
            return "";
        String str = "";
        if (type.toUpperCase().indexOf("VARCHAR") != -1) {
            str += "(255)";
        } else if (type.toUpperCase().indexOf("CHAR") != -1) {
            str += "(5)";
        } else {
            // 其他类型
        }
        return str;
    }

    private String[] genSourceColumnsInfo(DmpSyncingDatasourceType s, DmpSyncingDatasourceType t, String ft,
                                          String sTable, String sTable_, String _extract, String extractExpression, List<Map<String, Object>> rColumns,
                                          String sourceDbName) {
        String source_columns = "";
        String source_columns_hive_partition = "";
        String source_table = "<![CDATA[ (select ";
        String source_sql = "<![CDATA[ select ";
        String source_sql_hive_partition = "";
        String clean_rules = "";
        String check_rules = "";
        Map<String, String> relatedTypeMap = getRelatedTypeMap(s, t);
        int i = 0;
        for (Map<String, Object> m : rColumns) {

            String name = (String) m.get("name");
            String type = StringUtils.getStringValue(m.get("type"), "");//relatedTypeMap.get(StringUtils.getStringValue(m.get("type"), ""));
            String isPt = (String) m.get("isPt");
            if ("SQLSERVER".equals(s.getDatasourceType())) { //对sqlserver 修改
                if ("STRING".equals(type)) {
                    source_table += "REPLACE(REPLACE(REPLACE(" + name + ", CHAR(13), '<br1>'),CHAR(10),'<br2>'),',','<comma>') AS " + name;
                } else {
                    source_table += name;
                }
            } else {
//            	source_table += "replace(cast(" + name + " as char), char(10), '') as " + name;
                source_table += name;
            }
            if ("HIVE".equals(t.getDatasourceType()) && "1".equals(isPt)) {
                source_columns_hive_partition += name;
                source_sql_hive_partition += "cast(" + name + " as " + type + ") as ";
                if ("json".equals(ft)) {
                    source_sql_hive_partition += name.toLowerCase();
                } else {
                    source_sql_hive_partition += name;
                }
            } else {
                source_columns += name;
                if ("varchar".equalsIgnoreCase(type)) {
                    source_sql += "cast(" + name + " as string) as ";
                } else if ("char".equalsIgnoreCase(type)) {
                    source_sql += "cast(" + name + " as string) as ";
                } else if ("datetime".equalsIgnoreCase(type)) {
                    source_sql += "cast(" + name + " as timestamp) as ";
                } else {
                    source_sql += "cast(" + name + " as " + type + ") as ";
                }
                if ("json".equals(ft)) {
                    source_sql += name.toLowerCase();
                } else {
                    source_sql += name;
                }
            }
            source_table += ", ";
            if ("HIVE".equals(t.getDatasourceType()) && "1".equals(isPt)) {
                source_columns_hive_partition += ", ";
                source_sql_hive_partition += ", ";
            } else {
                source_columns += ", ";
                source_sql += ", ";
            }
            if (i == rColumns.size() - 1) {
                String loweredStr = source_sql.toLowerCase();
                boolean appjz = loweredStr.indexOf("etl_create_time,") == -1;
                boolean appUpd = loweredStr.indexOf("etl_update_time,") == -1;
                if (appjz) source_sql += "cast(current_timestamp() as string) as etl_create_time, ";
                if (appUpd) source_sql += "cast(current_timestamp() as string) as etl_update_time, ";
            }
            i++;
        }
        if ("HIVE".equals(t.getDatasourceType()) && StringUtils.isNotBlank(source_columns_hive_partition)) {
            //source_sql_hive_partition = source_sql_hive_partition.substring(0, source_sql_hive_partition.length()-2);
            //source_columns_hive_partition = source_columns_hive_partition.substring(0, source_columns_hive_partition.length()-2);
            source_sql += source_sql_hive_partition;
            source_columns += source_columns_hive_partition;
        }
        source_table = source_table.substring(0, source_table.length() - 2);
        if ("RDBMS".equals(s.getDatasourceCatecode())) {
            source_table += " from " + sourceDbName + "." + sTable_;
        } else {
            source_table += " from " + sTable;
        }
        if (!"full".equalsIgnoreCase(_extract) && StringUtils.isNotBlank(extractExpression)) {
            source_table += " " + extractExpression;
        }
        source_table += ") as t]]>";
        if ("FTP".equals(s.getDatasourceType()) || "HDFS".equals(s.getDatasourceType())) {
            sTable = "$$tmpTableName";
        }
        source_sql = source_sql.substring(0, source_sql.length() - 2);
        source_columns = source_columns.substring(0, source_columns.length() - 2);
        source_sql += " from " + sTable + "]]> ";
        return new String[]{source_table, source_columns, source_sql, clean_rules, check_rules};
    }

    private Map<String, String> getRelatedTypeMap(DmpSyncingDatasourceType sds, DmpSyncingDatasourceType tds) {
        /* 封装cast类型，直接根据key=源字段名获取；默认封装2kudu，除非源或目标库是hive库 */
        Map<String, Object> params = new HashMap();
        Map<String, String> typeMap = new HashMap();
        DmpSyncingDatasourceType entity = new DmpSyncingDatasourceType();
        if ("HIVE".equalsIgnoreCase(sds.getDatasourceType()) || "HIVE".equalsIgnoreCase(tds.getDatasourceType())) {
            entity.setDatasourceType("HIVE");
        } else { // 剩余情况均可认定为2kudu
            entity.setDatasourceType("KUDU");
        }

        /*params.put("sourceDsId", sds.getId());
        params.put("targetDsId", dmpSyncingDatasourceTypeService.findList(entity).get(0).getId());
        List<Map<String, Object>> dataTypeMapping = dmpTableFieldSchemaService.getDataTypeMapping(params);
        for (Map<String, Object> mapp : dataTypeMapping) {
            typeMap.put(StringUtils.toString(mapp.get("sourceType")),
                    StringUtils.getStringValue(mapp.get("targetType"), "STRING"));
        }*/
        return typeMap;
    }

    private String getSourceHdfsDir(String filePath, String fileName) {
        if (StringUtils.isNotBlank(fileName)) {
            boolean f1 = filePath.endsWith("/");
            boolean f2 = fileName.startsWith("/");
            if (f1 && f2) {
                fileName = fileName.substring(1, fileName.length());
            }
            if (!f1 && !f2) {
                filePath += "/";
            }
            return filePath + fileName;
        } else {
            return filePath;
        }
    }

    private String[] genTargetColumnsInfo(DmpSyncingDatasourceType s, DmpSyncingDatasourceType t, String tTable,
                                          String _targetBucketCounts, List<Map<String, Object>> wColumns) {
        String target_columns = "";
        String target_table_creation = "create table if not exists " + tTable + "(";
        String target_primary_columns = "";
        String target_partition_columns = "";
        String target_partition_columns_hive = "";
        String target_columns_hive_partition = "";
        String clean_rules = "";
        String check_rules = "";
        int i = 0;
        for (Map<String, Object> m : wColumns) {
            String name = (String) m.get("name");
            String type = (String) m.get("type");
            String comment = m.get("comment") == null ? "" : (String) m.get("comment");
            String isPk = (String) m.get("isPk");
            String isPt = (String) m.get("isPt");
            if ("HIVE".equals(t.getDatasourceType()) && "1".equals(isPt)) {
                target_columns_hive_partition += name;
            } else {
                target_columns += name;
            }

            if (!"1".equals(isPt) && "HIVE".equals(t.getDatasourceType())) {
                target_table_creation += name + " " + type;
                target_table_creation += " comment '" + comment + "'";
            } else if ("KUDU".equals(t.getDatasourceType())) {
                target_table_creation += name + " " + type;
                target_table_creation += "1".equals(isPk) ? " not null " : " null ";
                target_table_creation += " comment '" + comment + "'";
            } else if ("RDBMS".equals(t.getDatasourceCatecode())) {
                // POSTGRESQL支持不指定varchar的长度
                target_table_creation += name + " " + type + getTypeSize(type);
                target_table_creation += "1".equals(isPk) ? " not null " : " null ";
                target_table_creation += "comment '" + comment + "'";
            }
            if ("1".equals(isPk)) {
                target_primary_columns += name + ", ";
            }
            if ("1".equals(isPt)) {
                target_partition_columns += name + ", ";
                target_partition_columns_hive += name + " " + type + ", ";
            }
            String[] rls = this.appendRules(m.get("rules"), name, clean_rules, check_rules);//便利清洗规则
            clean_rules = rls[0];
            check_rules = rls[1];
            if ("HIVE".equals(t.getDatasourceType()) && "1".equals(isPt)) {
                target_columns_hive_partition += ", ";
            } else {
                target_columns += ", ";
            }
            if (!"1".equals(isPt) && "HIVE".equals(t.getDatasourceType())) {
                target_table_creation += ", ";
            } else if ("KUDU".equals(t.getDatasourceType())) {
                target_table_creation += ", ";
            } else if ("RDBMS".equals(t.getDatasourceCatecode())) {
                target_table_creation += ", ";
            }
            if (i == wColumns.size() - 1) {
                // 若源表自带etl_create_time、etl_update_time字段，则不进行追加
                String loweredStr = target_columns.toLowerCase();
                boolean appjz = loweredStr.indexOf("etl_create_time,") == -1;
                boolean appUpd = loweredStr.indexOf("etl_update_time,") == -1;
                if ("KUDU".equals(t.getDatasourceType())) {
                    if (appjz) target_table_creation += "etl_create_time string null comment 'etl_create_time', ";
                    if (appUpd) target_table_creation += "etl_update_time string null comment 'etl_update_time', ";
                } else if ("HIVE".equals(t.getDatasourceType())) {
                    if (appjz) target_table_creation += "etl_create_time string comment 'etl_create_time', ";
                    if (appUpd) target_table_creation += "etl_update_time string comment 'etl_update_time', ";
                } else if ("RDBMS".equals(t.getDatasourceCatecode())) {
                    // 未兼容常用关系型Microsoft Access, DM
                    if ("ORACLE".equals(t.getDatasourceType())) {
                        if (appjz) target_table_creation += "etl_create_time varchar2(30) comment 'etl_create_time', ";
                        if (appUpd) target_table_creation += "etl_update_time varchar2(30) comment 'etl_update_time', ";
                    } else {
                        if (appjz) target_table_creation += "etl_create_time varchar(30) comment 'etl_create_time', ";
                        if (appUpd) target_table_creation += "etl_update_time varchar(30) comment 'etl_update_time', ";
                    }
                }
                if (appjz) target_columns += "etl_create_time, ";
                if (appUpd) target_columns += "etl_update_time, ";

                if ("HIVE".equals(t.getDatasourceType()) && StringUtils.isNotBlank(target_columns_hive_partition)) {
                    target_columns += target_columns_hive_partition;
                }

                if (StringUtils.isNotBlank(target_primary_columns))
                    target_primary_columns = target_primary_columns.substring(0, target_primary_columns.length() - 2);
                if (StringUtils.isNotBlank(target_partition_columns_hive))
                    target_partition_columns_hive = target_partition_columns_hive.substring(0, target_partition_columns_hive.length() - 2);
                if (StringUtils.isNotBlank(target_partition_columns))
                    target_partition_columns = target_partition_columns.substring(0, target_partition_columns.length() - 2);
                if (StringUtils.isNotBlank(target_table_creation))
                    target_table_creation = target_table_creation.substring(0, target_table_creation.length() - 2);
                if (StringUtils.isNotBlank(target_primary_columns)) {
                    if ("KUDU".equals(t.getDatasourceType()) || "RDBMS".equals(t.getDatasourceCatecode())) {
                        target_table_creation += ", primary key (" + target_primary_columns + ") ";
                    }
                }
            }
            i++;
        }
        target_table_creation += ") ";
        if ("KUDU".equals(t.getDatasourceType())) {
            if (StringUtils.isNotBlank(target_partition_columns)) {
                target_table_creation += "partition by hash (" + target_partition_columns + ") ";
            } else {
                if (StringUtils.isNotBlank(target_primary_columns)) {
                    target_table_creation += "partition by hash (" + target_primary_columns + ") ";
                }
            }
            if (StringUtils.isNotBlank(_targetBucketCounts)) {
                target_table_creation += "partitions  " + _targetBucketCounts;
            }
            target_table_creation += " stored as kudu ";
        } else if ("HIVE".equals(t.getDatasourceType())) {
            if (StringUtils.isNotBlank(target_partition_columns)) {
                target_table_creation += "partitioned by (" + target_partition_columns_hive + ") ";
            }
        } else if ("RDBMS".equals(t.getDatasourceCatecode())) {
            // 分区，暂忽略关系型数据库的分区功能
        }
        target_columns = target_columns.substring(0, target_columns.length() - 2);
        return new String[]{target_columns, target_primary_columns, target_partition_columns,
                target_table_creation, clean_rules, check_rules};
    }

    private String[] appendRules(Object obj, String columnName, String clean_rules, String check_rules) {
        List<Map<String, Object>> rules = (List<Map<String, Object>>) obj;
        if (rules.size() > 0 && rules != null) {
            for (Map<String, Object> m : rules) {
                String method = (String) m.get("method");        // 规则方法
                String type = (String) m.get("type");            // 1:清洗规则;0:校验规则
                List<String> params = (ArrayList<String>) m.get("params");    // 规则参数列表，字符串
                String sub_rules = "\t<column_name name=\"" + columnName + "\">\r\n";
                sub_rules += "\t\t<rules_type method=\"" + method + "\">\r\n";
                if (params != null && params.size() > 0)
                    for (String rv : params) {
                        sub_rules += "\t\t\t<rules_value>" + rv + "</rules_value>\r\n";
                    }
                sub_rules += "\t\t</rules_type>\r\n";
                sub_rules += "\t</column_name>\r\n";
                if ("0".equals(type)) {
                    if (StringUtils.isBlank(check_rules))
                        sub_rules = "\r\n" + sub_rules;
                    check_rules += sub_rules;
                }
                if ("1".equals(type)) {
                    if (StringUtils.isBlank(clean_rules))
                        sub_rules = "\r\n" + sub_rules;
                    clean_rules += sub_rules;
                }
            }
        }
        return new String[]{clean_rules, check_rules};
    }

    /**
     * 数据运维-数据开发任务列表分页查询
     *
     * @return
     * @author Bellamy
     * @since 2021-01-13
     */
    @Override
    public PageInfoResponse<DataDevTaskListDto> queryDevTaskListPage(DataDevTaskListReq req) throws Exception {
        DmpProjectSystemInfo publishToProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(Long.valueOf(req.getProjectId()));
        //调用azkaban服务
        String azkabanApiUrl = publishToProjectSystemInfo.getAzkabanMonitorUrl();
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanApiUrl, redisTemplate);

        if (StringUtils.isNotBlank(req.getStatus())) {
            if (FlowInstanceEnum.running.getCode().equals(req.getStatus())) {
                req.setStatus(FlowInstanceEnum.running.getValue());
            } else if (FlowInstanceEnum.failed.getCode().equals(req.getStatus())) {
                req.setStatus(FlowInstanceEnum.failed.getValue());
            } else {
                //全部
                req.setStatus("");
            }
        }

        //查询实例最后执行状态
        List<Map> flowList = executionFlowsMapper.queryLastStatus(null, req.getStatus());
        for (Map strFlow : flowList) {
            String projectId = String.valueOf(strFlow.get("projectId"));
            String flowId = String.valueOf(strFlow.get("taskName"));
            //获取调度配置
            Map responseData = (Map) JSONObject.parse(azkabanApiUtils.getSchedule(projectId, flowId));
            if (responseData != null && responseData.size() > 0) {
                Map schedule = (Map) responseData.get("schedule");
                strFlow.put("scheduleId", schedule.get("scheduleId"));
                strFlow.put("cronExpression", schedule.get("cronExpression")); //调度周期
                //获取sla 是否已配置 状态
                if (null == schedule.get("scheduleId")) {
                    strFlow.put("slaStatus", "N");
                } else {
                    //获取sla
                    String resp = azkabanApiUtils.getSlaInfo(String.valueOf(schedule.get("scheduleId")));
                    if (StringUtils.isNotBlank(resp)) {
                        strFlow.put("slaStatus", "Y");
                    } else {
                        strFlow.put("slaStatus", "N");
                    }
                }
            }
        }

        PageInfoResponse<DataDevTaskListDto> pageInfoResponse = new PageInfoResponse<>();
        if (StringUtils.isNotBlank(req.getTreeIdOrName())) {
            //判断是否为整数 是整数返回true,否则返回false
            Pattern pattern = Pattern.compile("^[-\\+]?[\\d]*$");
            if (pattern.matcher(req.getTreeIdOrName().trim()).matches()) {
                req.setTaskId(req.getTreeIdOrName()); //id
                req.setTreeIdOrName(null);
            } else {
                req.setTreeIdOrName(req.getTreeIdOrName().trim());//任务名称
            }
        }
        //业务流程任务
        PageHelper.startPage(req.getPageNum(), req.getPageSize());
        List<DataDevTaskListDto> list = dmpDevelopTaskDao.queryDevTaskListPage(req);
        PageInfo<DataDevTaskListDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);

        List<DataDevTaskListDto> listObj = pageInfoResponse.getData().getList();
        if (flowList.size() > 0 && flowList != null) {
            for (DataDevTaskListDto str : listObj) { //业务流程任务
                for (Map strFlow : flowList) { //最后实例状态
                    if (CommonUtils.getAzkabanName(str.getTaskId()).equals(strFlow.get("taskName"))) {
                        str.setStatus(String.valueOf(strFlow.get("status"))); //实例执行最后状态
                        if (null != strFlow.get("cronExpression")) { //调度周期
                            str.setSchedulinCycle(String.valueOf(strFlow.get("cronExpression")));
                        }
                        if (null != strFlow.get("scheduleId")) {
                            str.setScheduleId(String.valueOf(strFlow.get("scheduleId")));
                        }
                        if (null != strFlow.get("slaStatus")) {
                            str.setSlaStatus(String.valueOf(strFlow.get("slaStatus")));
                        }
                    }
                }
            }
        }

        //查询实例最后执行状态
       /* List<Map> flowList = executionFlowsMapper.queryLastStatus(taskName.substring(1).split(","), req.getStatus());
        for (Map strFlow : flowList) {
            String projectId = String.valueOf(strFlow.get("projectId"));
            String flowId = String.valueOf(strFlow.get("taskName"));
            //获取调度配置
            Map responseData = (Map) JSONObject.parse(azkabanApiUtils.getSchedule(projectId, flowId));
            if (responseData != null && responseData.size() > 0) {
                Map schedule = (Map) responseData.get("schedule");
                strFlow.put("scheduleId", schedule.get("scheduleId"));
                strFlow.put("cronExpression", schedule.get("cronExpression")); //调度周期
                //获取sla 是否已配置 状态
                if (null == schedule.get("scheduleId")) {
                    strFlow.put("slaStatus", "N");
                } else {
                    JsonResult jsonResult = getSlaInfo(String.valueOf(schedule.get("scheduleId")), "");
                    Map s = (Map) jsonResult.getData();
                    if (s.size() > 0 && s != null) {
                        strFlow.put("slaStatus", "Y");
                    }
                }
            }
        }
        if (flowList.size() > 0 && flowList != null) {
            for (DataDevTaskListDto str : listObj) {
                for (Map strFlow : flowList) {
                    if (str.getTaskName().equals(strFlow.get("taskName"))) {
                        str.setStatus(String.valueOf(strFlow.get("status"))); //实例执行最后状态
                        if (null != strFlow.get("cronExpression")) { //调度周期
                            str.setSchedulinCycle(String.valueOf(strFlow.get("cronExpression")));
                        }
                        if (null != strFlow.get("scheduleId")) {
                            str.setScheduleId(String.valueOf(strFlow.get("scheduleId")));
                        }
                    }
                }
            }
        }*/

        return pageInfoResponse;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * com.ycxc.vmts.service.DmpDevelopTaskService#findList(com.ycxc.vmts.controller.bas.
     * bean.DmpDevelopTaskRequest, javax.servlet.http.HttpServletRequest)
     */
    @Override
    public com.jz.common.bean.PageInfoResponse<DmpDevelopTaskDto> findListWithPage(DmpDevelopTaskRequest dmpDevelopTaskRequest, HttpServletRequest httpRequest)
            throws Exception {
        com.jz.common.bean.PageInfoResponse<DmpDevelopTaskDto> pageInfoResponse = new com.jz.common.bean.PageInfoResponse<>();

        Map<String, Object> param = new HashMap<>();
        //ID
        if (dmpDevelopTaskRequest.getId() != null) {
            param.put("id", dmpDevelopTaskRequest.getId());
        }
        //tree ID
        if (dmpDevelopTaskRequest.getTreeId() != null) {
            param.put("treeId", dmpDevelopTaskRequest.getTreeId());
        }
        //数据源ID
        if (dmpDevelopTaskRequest.getDatasourceId() != null) {
            param.put("datasourceId", dmpDevelopTaskRequest.getDatasourceId());
        }
        //任务类型
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getTaskType())) {
            param.put("taskType", dmpDevelopTaskRequest.getTaskType());
        }
        //类型
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getType())) {
            param.put("type", dmpDevelopTaskRequest.getType());
        }
        //调度类型
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getScheduleType())) {
            param.put("scheduleType", dmpDevelopTaskRequest.getScheduleType());
        }
        //是否已提交
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getIsSubmit())) {
            param.put("isSubmit", dmpDevelopTaskRequest.getIsSubmit());
        }
        //描述
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getTaskDesc())) {
            param.put("taskDesc", dmpDevelopTaskRequest.getTaskDesc());
        }
        //脚本
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getScript())) {
            param.put("script", dmpDevelopTaskRequest.getScript());
        }
        //数据状态
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getDataStatus())) {
            param.put("dataStatus", dmpDevelopTaskRequest.getDataStatus());
        }
        //创建用户ID
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getCreateUserId())) {
            param.put("createUserId", dmpDevelopTaskRequest.getCreateUserId());
        }
        //数据创建时间起
        if (dmpDevelopTaskRequest.getCreateTimeStart() != null) {
            param.put("createTimeStart", dmpDevelopTaskRequest.getCreateTimeStart());
        }
        //数据创建时间止
        if (dmpDevelopTaskRequest.getCreateTimeEnd() != null) {
            param.put("createTimeEnd", dmpDevelopTaskRequest.getCreateTimeEnd());
        }
        //创建用户ID
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getUpdateUserId())) {
            param.put("updateUserId", dmpDevelopTaskRequest.getUpdateUserId());
        }
        //数据更新时间起
        if (dmpDevelopTaskRequest.getUpdateTimeStart() != null) {
            param.put("updateTimeStart", dmpDevelopTaskRequest.getUpdateTimeStart());
        }
        //数据更新时间止
        if (dmpDevelopTaskRequest.getUpdateTimeEnd() != null) {
            param.put("updateTimeEnd", dmpDevelopTaskRequest.getUpdateTimeEnd());
        }
        //校验状态:SUCCEED 成功, FAIL 失败
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getChkResult())) {
            param.put("chkResult", dmpDevelopTaskRequest.getChkResult());
        }
        //同步状态:SUCCEED 成功，FAIL 失败
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getSyncResult())) {
            param.put("syncResult", dmpDevelopTaskRequest.getSyncResult());
        }
        //最终校验时间起
        if (dmpDevelopTaskRequest.getChkTimeStart() != null) {
            param.put("chkTimeStart", dmpDevelopTaskRequest.getChkTimeStart());
        }
        //最终校验时间止
        if (dmpDevelopTaskRequest.getChkTimeEnd() != null) {
            param.put("chkTimeEnd", dmpDevelopTaskRequest.getChkTimeEnd());
        }
        //最终同步时间起
        if (dmpDevelopTaskRequest.getSyncTimeStart() != null) {
            param.put("syncTimeStart", dmpDevelopTaskRequest.getSyncTimeStart());
        }
        //最终同步时间止
        if (dmpDevelopTaskRequest.getSyncTimeEnd() != null) {
            param.put("syncTimeEnd", dmpDevelopTaskRequest.getSyncTimeEnd());
        }
        //
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getFlowHeader())) {
            param.put("flowHeader", dmpDevelopTaskRequest.getFlowHeader());
        }
        //
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getFlowJson())) {
            param.put("flowJson", dmpDevelopTaskRequest.getFlowJson());
        }
        //
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getVersion())) {
            param.put("version", dmpDevelopTaskRequest.getVersion());
        }
        //
        if (dmpDevelopTaskRequest.getIsGziped() != null) {
            param.put("isGziped", dmpDevelopTaskRequest.getIsGziped());
        }
        //源数据库ID
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getSourceDbId())) {
            param.put("sourceDbId", dmpDevelopTaskRequest.getSourceDbId());
        }
        //源数据库名称
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getSourceDbName())) {
            param.put("sourceDbName", dmpDevelopTaskRequest.getSourceDbName());
        }
        //源数据表名称
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getSourceTableName())) {
            param.put("sourceTableName", dmpDevelopTaskRequest.getSourceTableName());
        }
        //目标数据库名称
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getTargetDbName())) {
            param.put("targetDbName", dmpDevelopTaskRequest.getTargetDbName());
        }
        //目标数据表名称
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getTargetTableName())) {
            param.put("targetTableName", dmpDevelopTaskRequest.getTargetTableName());
        }
        //项目ID
        if (dmpDevelopTaskRequest.getProjectId() != null) {
            param.put("projectId", dmpDevelopTaskRequest.getProjectId());
        }
        //创建用户名称
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getCreateUserName())) {
            param.put("createUserName", dmpDevelopTaskRequest.getCreateUserName());
        }
        //任务名称
        if (!StringUtils.isEmpty(dmpDevelopTaskRequest.getName())) {
            param.put("name", dmpDevelopTaskRequest.getName());
        }
        //未删除数据
        param.put("dataStatus", "1");

        String obderByStr = dmpDevelopTaskRequest.takeOrderByStr(DmpDevelopTaskRequest.class);
        Page page = null;
        if (obderByStr == null) {
            page = PageHelper.startPage(dmpDevelopTaskRequest.getPageNum(), dmpDevelopTaskRequest.getPageSize());
        } else {
            page = PageHelper.startPage(dmpDevelopTaskRequest.getPageNum(), dmpDevelopTaskRequest.getPageSize(), obderByStr);
        }

        List<DmpDevelopTask> list = dmpDevelopTaskDao.findList(param);

        List<DmpDevelopTaskDto> dtos = MyDmpDevelopTaskConverter.INSTANCE().domain2dto(list);

        PageInfo<DmpDevelopTaskDto> pageInfo = new PageInfo<>(dtos);
        pageInfo.setTotal(page.getTotal());

        pageInfoResponse.setCode(StatuConstant.SUCCESS_CODE);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 数据运维-执行任务
     *
     * @return
     * @author Bellamy
     */
    @Override
    public JsonResult runTaskByTaskId(String taskId) throws Exception {
        return offlineSynchService.taskRunNowByTaskId(taskId);
    }

    /**
     * 数据运维-数据开发实例列表分页查询
     *
     * @return
     * @author Bellamy
     * @since 2021-01-14
     */
    @Override
    public PageInfoResponse<DataDevExamplesListDto> queryDevExamplesListPage(DataDevExamplesListReq req) throws Exception {
        PageInfoResponse<DataDevExamplesListDto> pageInfoResponse = new PageInfoResponse<>();

        String taskName = ""; //任务名称
        //查询离线任务信息
        Map params = new HashMap();
        params.put("projectId", req.getProjectId());
        if (StringUtils.isNotBlank(req.getTreeName())) {
            params.put("taskName", req.getTreeName().trim());
        }
        if (StringUtils.isNotBlank(req.getTreeName())) {
            params.put("treeId", req.getTaskId());
        }
        List<DataDevTaskListDto> treeList = dmpDevelopTaskDao.queryTaskTreeInfo(params);
        if (treeList.size() > 0 && treeList != null) {
            for (DataDevTaskListDto treeDto : treeList) {
                if (StringUtils.isNotBlank(treeDto.getTaskName())) {
                    //taskName += "," + treeDto.getTaskName();
                    String azkabanJobId = CommonUtils.getAzkabanName(treeDto.getTaskId());
                    taskName += "," + azkabanJobId;
                    treeDto.setAzkabanJobId(azkabanJobId);
                }
            }
            //通过任务名称，去查询开发实例
            PageHelper.startPage(req.getPageNum(), req.getPageSize());
            Map saveParams = new HashMap();
            if (StringUtils.isNotBlank(req.getStartTime()) && StringUtils.isNotBlank(req.getEndTime())) { //业务时间范围
                //String[] cretime = req.getBusinessTime().split("-");
                saveParams.put("startTime", req.getStartTime().trim() + " 00:00:00");
                saveParams.put("endTime", req.getEndTime().trim() + " 23:59:59");
            } else if (StringUtils.isNotBlank(req.getCreTimeType())) {
                if ("01".equals(req.getCreTimeType())) { //昨天
                    saveParams.put("businessTime", DateUtils.getYesterdayStr());
                }
                if ("02".equals(req.getCreTimeType())) { //前天
                    saveParams.put("businessTime", DateUtils.getBeforeYesterdayStr());
                }
                if ("03".equals(req.getCreTimeType())) { //全部
                    saveParams.put("businessTime", null);
                }
            } else {
                //默认当天日期
                saveParams.put("businessTime", DateUtils.currentDate());
                //saveParams.put("businessTime", "2021-01-07");
            }
            saveParams.put("taskName", taskName.substring(1).split(","));
            List<DataDevExamplesListDto> list = executionFlowsMapper.queryDevExamplesListPage(saveParams);
            if (list.size() > 0) {
                for (DataDevTaskListDto treeDto : treeList) {
                    for (DataDevExamplesListDto dto : list) {
                        if (treeDto.getAzkabanJobId().equals(dto.getTaskName())) {
                            dto.setTaskType(treeDto.getType());
                            BigDecimal cost = new BigDecimal(dto.getRunTime()).divide(new BigDecimal(1000), 2, BigDecimal.ROUND_HALF_UP);
                            dto.setRunTime(cost + "s");
                            //JsonMapper.fromJsonString(dto.getFlowData(), Map.class);
                        }
                    }
                }
            }
            PageInfo<DataDevExamplesListDto> pageInfo = new PageInfo<>(list);
            pageInfoResponse.setCode(ResultCode.SUCCESS);
            pageInfoResponse.setMessage("查询成功");
            pageInfoResponse.setData(pageInfo);
        }
        return pageInfoResponse;
    }

    /**
     * 根据treeId取得最新版提交的同步脚本文件名（不含后缀）及版本信息
     */
    @Override
    public String getConfigFileNameNotSuffix4Published(Long treeId) throws Exception {
        DmpModuleOperateLog log = getLastVersion4SubmitSyncInfo(treeId.intValue());
        String fileName = XmlUtils.getPropertyValue(log.getOperateLog(), "name");
        return fileName + "@" + log.getRemark();
    }

    private DmpModuleOperateLog getLastVersion4SubmitSyncInfo(Integer treeId) throws Exception {
        DmpModuleOperateLog o = null;
        try {
            o = dmpModuleOperateLogService.getLastOperateLog(treeId, ModuleLogEnum.VERSION_SYNC_XML);
            // #######经与宏涛确认忽略历史数据，跳过初始化。
            /*if (null == o) {
                // 未找到历史版本，考虑到历史离线同步数据存在无版本记录的情况，初始化版本信息
                DmpTaskSchedule po = new DmpTaskSchedule();
                po.setTreeId(task.getTreeId());
                po.setProjectId(task.getProjectId());
                String xmlContent = convert2SyncXmlContent(po);
                o = dmpModuleOperateLogService.writeSubmitXmlVersion(task.getTreeId(), xmlContent);
            }*/
            if (null == o || !StringUtils.hasLength(o.getRemark())) {
                throw new Exception("未找到历史版本信息");
            }
        } catch (Exception e) {
            logger.error("获取离线同步xml文件名及版本信息异常：" + e.getMessage(), e);
        }
        return o;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * com.ycxc.vmts.service.DmpDevelopTaskService#add(com.ycxc.vmts.entity.DmpDevelopTask,
     * javax.servlet.http.HttpServletRequest)
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseBeanResponse<DmpDevelopTask> add(DmpDevelopTask dmpDevelopTask, HttpServletRequest httpRequest) throws Exception {
        BaseBeanResponse<DmpDevelopTask> baseBeanResponse = new BaseBeanResponse<>();

        //参数校验
        String taskName = dmpDevelopTask.getName();
        if (StringUtils.isEmpty(taskName)) {
            baseBeanResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
            baseBeanResponse.setMessage("任务名称不能为空");
            return baseBeanResponse;
        }

        //任务类型校验
        String taskType = dmpDevelopTask.getTaskType();
        String treeType = "";
        if (StringUtils.isEmpty(taskType)) {
            baseBeanResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
            baseBeanResponse.setMessage("任务类型不能为空");
            return baseBeanResponse;
        } else if (taskType.equals(CommConstant.TASK_TYPE_DEVELOP)) {
            treeType = CommConstant.TREE_TYPE_DEVELOP;
        } else if (taskType.equals(CommConstant.TASK_TYPE_DEVSHELL)) {
            treeType = CommConstant.TREE_TYPE_DEVSHELL;
        } else if (taskType.equals(CommConstant.TASK_TYPE_DEVSQL)) {
            treeType = CommConstant.TREE_TYPE_DEVSQL;
        } else {
            baseBeanResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
            baseBeanResponse.setMessage("任务类型错误");
            return baseBeanResponse;
        }

        //添加树叶子节点
        DmpNavigationTree tree = new DmpNavigationTree();
        tree.setName(taskName);
        tree.setParentId(dmpDevelopTask.getParentId());
        tree.setProjectId(dmpDevelopTask.getProjectId());
        tree.setCategory(CommConstant.TREE_DEVELOP_TASK);
        tree.setType(treeType);
        tree.setIsLevel("1");
        tree.setIsEnable("1");
        tree.setDataStatus("1");
        tree.setTreeSort(dmpNavigationTreeDao.getMaxSortById(tree));

        int cnt = dmpNavigationTreeDao.countTreeByName(tree);
        if (cnt > 0) {
            baseBeanResponse.setCode(StatuConstant.CODE_DATA_EXISTED);
            baseBeanResponse.setMessage("当前项目已存在同名的任务名称");
            return baseBeanResponse;
        }
        dmpNavigationTreeDao.insert(tree);

        //将新增任务保存到数据库
        dmpDevelopTask.setTaskType(taskType);
        dmpDevelopTask.setType("W");
        dmpDevelopTask.setIsSubmit("0");
        dmpDevelopTask.setScheduleType("2");
        dmpDevelopTask.setDataStatus("1");
        dmpDevelopTask.setTreeId(tree.getId().intValue());

        int revision = 1;
        ObjectNode flowHeaderNode = objectMapper.createObjectNode();
        flowHeaderNode.put("name", dmpDevelopTask.getName());
        flowHeaderNode.put("description", dmpDevelopTask.getTaskDesc());
        flowHeaderNode.put("revision", revision);
        dmpDevelopTask.setFlowHeader(flowHeaderNode.toString());

        ObjectNode scriptNode = objectMapper.createObjectNode();
        scriptNode.put("id", "canvas");
        scriptNode.put("resourceId", "canvas");
        ObjectNode stencilSetNode = objectMapper.createObjectNode();
        stencilSetNode.put("namespace", "http://b3mn.org/stencilset/bpmn2.0#");
        scriptNode.put("stencilset", stencilSetNode);
        byte[] data = null;
        try {
            data = scriptNode.toString().getBytes("utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        //dmpDevelopTask.setData(data);

        // 创建人
        dmpDevelopTask.setCreateUserId(Optional.ofNullable(getHttpRequestUserId(httpRequest)).orElse(0).toString());
        // 创建时间
        dmpDevelopTask.setCreateTime(new Date());

        //版本设置
        //获取已存在版本
        String version = CodeGeneratorUtils.generatorNextTaskVesion(null);
        dmpDevelopTask.setVersion(version);

        dmpDevelopTaskDao.insert(dmpDevelopTask);

        baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
        baseBeanResponse.setMessage("新增成功");
        baseBeanResponse.setData(dmpDevelopTask);

        return baseBeanResponse;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * com.ycxc.vmts.service.DmpDevelopTaskService#findByTreeId(com.ycxc.vmts.controller.
     * bean.DmpDevelopTaskRequest, javax.servlet.http.HttpServletRequest)
     */
    @Override
    public BaseBeanResponse<DmpDevelopTaskDto> findByTreeId(Integer treeId, HttpServletRequest httpRequest)
            throws Exception {
        BaseBeanResponse<DmpDevelopTaskDto> baseBeanResponse = new BaseBeanResponse<>();

        DmpDevelopTask dmpDevelopTask = dmpDevelopTaskDao.selectTaskInfoByParam(treeId);

        //dmpDevelopTask.setScript(new String(dmpDevelopTask.getData(), "utf-8"));

        DmpDevelopTaskDto dmpDevelopTaskDto = MyDmpDevelopTaskConverter.INSTANCE().domain2dto(dmpDevelopTask);

        baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
        baseBeanResponse.setMessage("查询成功");
        baseBeanResponse.setData(dmpDevelopTaskDto);

        return baseBeanResponse;
    }

    /**
     * 任务修改
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseBeanResponse<DmpDevelopTask> edit(DmpDevelopTask dmpDevelopTask, HttpServletRequest httpRequest)
            throws Exception {

        BaseBeanResponse<DmpDevelopTask> baseBeanResponse = new BaseBeanResponse<DmpDevelopTask>();

        // 修改人
        dmpDevelopTask.setUpdateUserId(getHttpRequestUserId(httpRequest).toString());
        // 修改时间
        dmpDevelopTask.setUpdateTime(new Date());

        // 版本设置
        // 获取已存在版本
        DmpDevelopTask dmpDevelopTaskDb = dmpDevelopTaskDao.selectTaskById(dmpDevelopTask.getId().toString());

        //校验脚本内容是否有变动，变动则允许修改，否则不允许修改
        if (!StringUtils.isEmpty(dmpDevelopTaskDb.getScript()) && dmpDevelopTaskDb.getScript().equals(dmpDevelopTask.getScript())) {
            baseBeanResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
            baseBeanResponse.setMessage("脚本内容没有任何变化，无效的操作！");
            return baseBeanResponse;
        }

        String version = CodeGeneratorUtils.generatorNextTaskVesion(dmpDevelopTaskDb.getVersion());
        dmpDevelopTask.setVersion(version);

        //dmpDevelopTask.setData(dmpDevelopTask.getScript().getBytes("utf-8"));

        dmpDevelopTaskDao.update(dmpDevelopTask);

        DmpDevelopTask dmpDevelopTaskDb2 = dmpDevelopTaskDao.selectTaskById(dmpDevelopTask.getId().toString());

        DmpDevelopTaskHistory dmpDevelopTaskHistory = MyDmpDevelopTaskHistoryConverter.INSTANCE().task2history(dmpDevelopTaskDb2);

        // 保存版本为版本数据
        dmpDevelopTaskHistoryService.add(dmpDevelopTaskHistory, httpRequest);

        baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
        baseBeanResponse.setMessage("修改成功");
        baseBeanResponse.setData(dmpDevelopTaskDb2);

        return baseBeanResponse;
    }

    /**
     * 任务流程发布
     */
    @Override
    public BaseResponse flowSubmit(Long treeId, HttpServletRequest httpRequest)
            throws Exception {

        BaseResponse baseResponse = new BaseResponse();

        // 参数校验
        if (treeId == null) {
            baseResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
            baseResponse.setMessage("treeId不能为空");
            return baseResponse;
        }

        DmpDevelopTask dmpDevelopTask = dmpDevelopTaskDao.get(treeId);

        if (!CommConstant.TASK_TYPE_DEVELOP.equals(dmpDevelopTask.getTaskType())) {
            baseResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
            baseResponse.setMessage("该任务不是工作流开发任务！");
            return baseResponse;
        }

        FlowPro flowPro = MyDmpDevelopTaskConverter.INSTANCE().task2flowpro(dmpDevelopTask);

        DmpNavigationTree dmpNavigationTree = dmpNavigationTreeDao.queryById(treeId.intValue());
        Long projectId = Long.parseLong(dmpNavigationTree.getProjectId().toString());
        flowPro.setPublishedToProjectId(projectId);
        //提交发布任务流程
        BaseResponse baseResponsePublish = flowService.publishFlow(flowPro);
        if (StatuConstant.SUCCESS_CODE.equals(baseResponsePublish.getCode())) {
            flowPro.setIsSubmit("1");
            //保存流程数据
            DmpDevelopTask queryDmpDevelopTask = dmpDevelopTaskDao.get(treeId);

            DmpDevelopTask updateDevelopTask = new DmpDevelopTask();
            updateDevelopTask.setId(queryDmpDevelopTask.getId());

            try {
                updateDevelopTask.setFlowJson(flowPro.getFlowJson());
                updateDevelopTask.setData(GZIPUtils.gzipString(flowPro.getFlowJson(), "utf-8"));
            } catch (IOException e) {
                e.printStackTrace();
            }
            updateDevelopTask.setIsSubmit(flowPro.getIsSubmit());
            updateDevelopTask.setIsGziped(1);

            //设置发布版本
            updateDevelopTask.setPublishVersion(queryDmpDevelopTask.getVersion());
            dmpDevelopTaskDao.update(updateDevelopTask);

            baseResponse.setCode(StatuConstant.SUCCESS_CODE);
            baseResponse.setMessage("工作流程保存发布成功");
        } else {
            baseResponse.setCode(StatuConstant.SUCCESS_CODE);
            baseResponse.setMessage(baseResponsePublish.getMessage());
        }

        return baseResponse;
    }

    /**
     * 根据treeId获取离线任务xmlFileName
     */
    @Override
    public String getExecXmlFileName(Long syncTaskTreeId) throws Exception {
        DmpDevelopTask dmpDevelopTask = dmpDevelopTaskDao.selectTaskInfoByParam(syncTaskTreeId);
        if (!dmpDevelopTask.getTaskType().equals(CommConstant.TASK_TYPE_OFFLINE)) {
            throw new RuntimeException("非离线任务！");
        }
        String xmlContent = convert2SyncXmlContent(dmpDevelopTask);

        String xmlFileName = XmlUtils.getPropertyValue(xmlContent, "name");

        return xmlFileName;
    }

    /**
     * 运行任务
     */
    @Override
    public BaseBeanResponse<Integer> taskAzkabanRun(Long treeId, HttpServletRequest httpRequest) throws Exception {

        BaseBeanResponse<Integer> baseBeanResponse = new BaseBeanResponse<Integer>();

        DmpDevelopTask developTask = dmpDevelopTaskDao.get(treeId);
        //先发布任务
        Integer execId = publishAndExecute(developTask);

        if (execId != null) {
            baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
            baseBeanResponse.setMessage("立即运行成功");
            baseBeanResponse.setData(execId);
        } else {
            baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
            baseBeanResponse.setMessage("立即运行失败");
        }

        return baseBeanResponse;
    }

    /**
     * 发布流程
     */
    private Integer publishAndExecute(DmpDevelopTask dmpDevelopTask) throws Exception {
        Integer taskId = dmpDevelopTask.getId(); //任务id
        Integer projectId = dmpDevelopTask.getProjectId(); //项目id
        String treeName = dmpDevelopTask.getName(); //任务流程名称

        //获取项目配置信息
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("projectId", projectId);
        List<DmpProjectConfigInfoDto> list = dmpProjectConfigInfoMapper.findList(param);
        if (CollectionUtils.isEmpty(list)) {
            throw new RuntimeException("项目没有设置配置信息，请联系管理员！");
        }
        DmpProjectConfigInfoDto dmpProjectConfigInfoDto = list.get(0);

        //校验任务类型并配置参数
        String taskType = dmpDevelopTask.getTaskType();
        String azkabanExectorTaskExec = "";
        String taskAlias = "";
        if (StringUtils.isEmpty(taskType)) {
            throw new RuntimeException("任务的任务类型为空！");
        } else if (taskType.equals(CommConstant.TASK_TYPE_DEVSHELL)) {
            azkabanExectorTaskExec = dmpProjectConfigInfoDto.getDmpPublicConfigInfoDto().getAzkabanExectorShellExec(); //执行shell任务命令
            taskAlias = "shell";
        } else if (taskType.equals(CommConstant.TASK_TYPE_DEVSQL)) {
            azkabanExectorTaskExec = dmpProjectConfigInfoDto.getDmpPublicConfigInfoDto().getAzkabanExectorSqlExec(); //执行sql任务命令
            taskAlias = "sql";
        } else {
            throw new RuntimeException("该任务类型不能调用此方法发布，请检查任务类型！");
        }

        String azkabanLocalTaskFilePath = dmpProjectConfigInfoDto.getDmpPublicConfigInfoDto().getAzkabanLocalTaskFilePath(); //文件路径
        String azkabanMonitorUrl = dmpProjectConfigInfoDto.getDmpPublicConfigInfoDto().getAzkabanMonitorUrl();//AZKABAN WEB服务地址

        String azkabanJobCommand = FlowParseTool.generateExecutorToolCommand(taskId, treeName, true);

        /**
         * 当前任务生成文件存放根路径
         */
        String localTaskPath = azkabanLocalTaskFilePath + "/" + projectId + "/local/" + taskId;
        File localTaskFile = new File(localTaskPath);
        if (!localTaskFile.exists()) {
            localTaskFile.mkdirs();
        }

        String localTaskSourcePath = localTaskPath + "/source/";
        File localTaskSourceFile = new File(localTaskSourcePath);
        if (!localTaskSourceFile.exists()) {
            localTaskSourceFile.mkdirs();
        }

        String localTaskExecArgsPath = localTaskPath + "/execArgs/";
        File localTaskExecArgsFile = new File(localTaskExecArgsPath);
        if (!localTaskExecArgsFile.exists()) {
            localTaskExecArgsFile.mkdirs();
        }

        String localTaskZipPath = localTaskPath + "/target/";
        File localTaskZipFile = new File(localTaskZipPath);
        if (!localTaskZipFile.exists()) {
            localTaskZipFile.mkdirs();
        }

        //先删除上次创建的文件 ---每次都会新建
        FileUtils.deleteDirectory(localTaskPath);


        List<String> contents = new ArrayList<>();

        // 子流程类型
        contents.add("type=command");
        contents.add(azkabanJobCommand);

        // 生成job文件
        String jobFileAbsolutePath = localTaskSourcePath + treeName + ".job";
        FileUtils.write(jobFileAbsolutePath, contents);

        String localZipTargetFileName = treeName + ".zip";
        ZipUtils.zip(localTaskSourcePath, localTaskZipPath, localZipTargetFileName);

        //上传到azkaban todo
        //上次zip包到azkaban
        String localTaskZipAbsolutePath = localTaskZipPath + "/" + localZipTargetFileName;
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanMonitorUrl, redisTemplate);
        //return azkabanApiUtils.loginCreateProjectuploadZipAndExecute("jz_localflow_" + taskAlias + "_" + projectId, "local_" + taskAlias + "_project", localTaskZipAbsolutePath, treeName);
        return azkabanApiUtils.loginCreateProjectuploadZipAndExecute("jz_workflow_new_" + projectId, "local_" + taskAlias + "_project", localTaskZipAbsolutePath, treeName);
    }

    /**
     * 停止任务
     */
    @Override
    public BaseBeanResponse<String> taskAzkabanStop(Long treeId, HttpServletRequest httpRequest) throws Exception {
        // TODO Auto-generated method stub
        BaseBeanResponse<String> baseBeanResponse = new BaseBeanResponse<String>();

        DmpDevelopTask dmpDevelopTask = dmpDevelopTaskDao.get(treeId);
        if (dmpDevelopTask == null) {
            baseBeanResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
            baseBeanResponse.setMessage("任务不存在");
            return baseBeanResponse;
        }

        Integer projectId = dmpDevelopTask.getProjectId();
        String treeName = dmpDevelopTask.getName();

        //获取项目配置信息
        Map<String, Object> param = new HashMap<String, Object>();
        List<DmpProjectConfigInfoDto> list = dmpProjectConfigInfoMapper.findList(param);
        if (CollectionUtils.isEmpty(list)) {
            throw new RuntimeException("项目没有设置配置信息，请联系管理员！");
        }
        DmpProjectConfigInfoDto dmpProjectConfigInfoDto = list.get(0);

        //校验任务类型并配置参数
        String taskType = dmpDevelopTask.getTaskType();
        String taskAlias = "";
        if (StringUtils.isEmpty(taskType)) {
            throw new RuntimeException("任务的任务类型为空！");
        } else if (taskType.equals(CommConstant.TASK_TYPE_DEVSHELL)) {
            taskAlias = "shell";
        } else if (taskType.equals(CommConstant.TASK_TYPE_DEVSQL)) {
            taskAlias = "sql";
        } else {
            throw new RuntimeException("该任务类型不能调用此方法停止，请检查任务类型！");
        }

        String azkabanMonitorUrl = dmpProjectConfigInfoDto.getDmpPublicConfigInfoDto().getAzkabanMonitorUrl();//AZKABAN WEB服务地址

        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanMonitorUrl, redisTemplate);

        String exeIdsStr = azkabanApiUtils.stopFlow("jz_localflow_" + taskAlias + "_" + projectId, treeName);

        baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
        baseBeanResponse.setMessage("停止成功");
        baseBeanResponse.setData(exeIdsStr);

        return baseBeanResponse;
    }

    /**
     * SHELL/SQL版本发布
     */
    @Override
    public BaseResponse taskPublish(Long treeId, HttpServletRequest httpRequest) throws Exception {

        BaseResponse baseResponse = new BaseResponse();

        DmpDevelopTask dmpDevelopTask = dmpDevelopTaskDao.get(treeId);

        // 修改人
        dmpDevelopTask.setUpdateUserId(getHttpRequestUserId(httpRequest).toString());
        // 修改时间
        dmpDevelopTask.setUpdateTime(new Date());
        //设置发布版本
        dmpDevelopTask.setPublishVersion(dmpDevelopTask.getVersion());

        dmpDevelopTaskDao.update(dmpDevelopTask);

        baseResponse.setCode(StatuConstant.SUCCESS_CODE);
        baseResponse.setMessage("发布成功");

        return baseResponse;
    }

    /**
     * 软删除任务
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse softDeleteByTreeId(Integer treeId, HttpServletRequest httpRequest) throws Exception {
        BaseResponse baseResponse = new BaseResponse();

        DmpDevelopTask developTask = dmpDevelopTaskDao.get(treeId.longValue());
        String taskType = developTask.getTaskType();
        if (taskType.equals(CommConstant.TASK_TYPE_DEVELOP)) {
            //取消发布的任务
            BaseResponse baseResponseAzkaban = flowService.deleteAzkabanFlow(developTask);
            if (baseResponseAzkaban.getCode().equals(StatuConstant.FAILURE_CODE)) {
                throw new RuntimeException("azkaban取消发布任务失败");
            }
        } else if (taskType.equals(CommConstant.TASK_TYPE_DEVSHELL) || taskType.equals(CommConstant.TASK_TYPE_DEVSQL)) {
            //什么也不做
        } else {
            baseResponse.setCode(StatuConstant.CODE_DATA_NOTMEET);
            baseResponse.setMessage("任务类型不适合调用该方法");
            return baseResponse;
        }

        //软删除树
        DmpNavigationTree dmpNavigationTree = new DmpNavigationTree();
        dmpNavigationTree.setId(treeId);
        dmpNavigationTree.setDataStatus("0");
        dmpNavigationTreeDao.update(dmpNavigationTree);

        //软删除任务
        developTask.setDataStatus("0");
        dmpDevelopTaskDao.update(developTask);

        baseResponse.setCode(StatuConstant.SUCCESS_CODE);
        baseResponse.setMessage("软删除成功");

        return baseResponse;
    }

    /**
     * 获取执行实例的日志详情
     *
     * @return
     * @author Bellamy
     * @since 2021-02-03
     */
    @Override
    public JsonResult queryExamplesLogByExecId(String execId, String projectId) throws Exception {
        DmpProjectSystemInfo publishToProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(Long.valueOf(projectId));

        //调用azkaban服务
        String azkabanApiUrl = publishToProjectSystemInfo.getAzkabanMonitorUrl();
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanApiUrl, redisTemplate);
        Map result = (Map) JSONObject.parse(azkabanApiUtils.getInstanceLog(execId));
        if (result.containsKey("data")) {
            return JsonResult.ok(result);
        }
        return JsonResult.error(ResultCode.OPERATION_DATA_NO_EXIST, String.valueOf(result.get("error")));
    }

    /**
     * 运维大屏--获取任务状态
     *
     * @param projectId
     * @return
     * @author Bellamy
     * @since 2021-02-22
     */
    @Override
    public JsonResult getTaskStatus(String projectId) throws Exception {
        Map<String, Integer> taskStatus = new HashMap<>();
        //RUNNING, FAILED,SUCCESS
        //实时任务状态
        Map<String, Object> realTimeTask = dmpRealtimeSyncInfoDao.queryTaskStatus(projectId);
        Integer realTimeRunNum = 0;
        Integer realTimeFailedNum = 0;
        if (realTimeTask != null && realTimeTask.size() > 0) {
            realTimeRunNum = Integer.valueOf(String.valueOf(realTimeTask.get("running")));
            realTimeFailedNum = Integer.valueOf(String.valueOf(realTimeTask.get("failed")));
        }
        DataDevTaskListReq req = new DataDevTaskListReq();
        req.setProjectId(projectId);
        List<DataDevTaskListDto> list = dmpDevelopTaskDao.queryDevTaskListPage(req);
        String taskName = "";
        if (list.size() > 0 && list != null) {
            for (DataDevTaskListDto str : list) {
                //taskName += "," + str.getTaskName();
                taskName += "," + CommonUtils.getAzkabanName(str.getTaskId());
            }
        }

        //业务流程实例状态 :30-正在运行,50-运行成功,60-kill任务,70-运行失败
        Map<String, Object> flowStatus = executionFlowsMapper.queryTaskInstanceStatus(taskName.substring(1).split(","));
        Integer flowRunNum = 0;
        Integer flowSuccessNum = 0;
        Integer flowFailedNum = 0;
        Integer flowManualNum = 0;
        if (flowStatus != null && flowStatus.size() > 0) {
            flowRunNum = Integer.valueOf(String.valueOf(flowStatus.get("running")));
            flowSuccessNum = Integer.valueOf(String.valueOf(flowStatus.get("success")));
            flowFailedNum = Integer.valueOf(String.valueOf(flowStatus.get("failed")));
            flowManualNum = Integer.valueOf(String.valueOf(flowStatus.get("manualNum")));
        }


        taskStatus.put("running", realTimeRunNum + flowRunNum);
        taskStatus.put("success", flowSuccessNum);
        taskStatus.put("failed", realTimeFailedNum + flowFailedNum);
        taskStatus.put("manual", flowManualNum);
        return JsonResult.ok(taskStatus);
    }

    /**
     * 设置SLA
     *
     * @return
     * @author Bellamy
     * @since 2021-02-03
     */
    @Override
    public JsonResult setSla(SetSlaReq req) throws Exception {
        Map<String, Object> params = new HashMap<>();
        Long projectId = Long.valueOf(req.getProjectId()); //项目id
        DmpProjectSystemInfo publishToProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(projectId);

        //调用azkaban服务
        String azkabanApiUrl = publishToProjectSystemInfo.getAzkabanMonitorUrl();
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanApiUrl, redisTemplate);
        azkabanApiUtils.saveSla(req);
        return JsonResult.ok();
    }

    /**
     * 运维大屏--任务完成情况
     *
     * @return
     * @author Bellamy
     * @since 2021-03-08
     */
    @Override
    public JsonResult getTaskStatusByHours(String projectId, String taskStatus) throws Exception {
        Map param = new HashMap();
        param.put("projectId", projectId);
        String taskName = "";
        List<DataDevTaskListDto> list = dmpDevelopTaskDao.queryTaskTreeInfo(param);
        for (DataDevTaskListDto str : list) {
            taskName += "," + str.getTaskName();
        }
        String[] taskNames = taskName.substring(1).split(",");

        String nowTime = DateUtils.currentDate(); //今天
        String yesterdayTime = DateUtils.getYesterdayStr(); //昨天
        List<String> nowList = new ArrayList<>();
        Map<String, Object> nowData = executionFlowsMapper.getTaskStatus(nowTime, taskStatus, taskNames);
        if (nowData == null || nowData.size() == 0) {
            for (int i = 0; i < 24; i++) {
                nowList.add("0");
            }
        } else {
            for (Map.Entry<String, Object> entry : nowData.entrySet()) {
                nowList.add(String.valueOf(entry.getValue()));
            }
        }

        List<String> yesterdayList = new ArrayList<>();
        Map<String, Object> yesterdayData = executionFlowsMapper.getTaskStatus(yesterdayTime, taskStatus, taskNames);
        if (yesterdayData == null || yesterdayData.size() == 0) {
            for (int i = 0; i < 24; i++) {
                yesterdayList.add("0");
            }
        } else {
            for (Map.Entry<String, Object> entry : yesterdayData.entrySet()) {
                yesterdayList.add(String.valueOf(entry.getValue()));
            }
        }
        //历史平均
        List<String> historyAvgList = new ArrayList<>();
        Map<String, Object> historyAvgData = executionFlowsMapper.getTaskStatus("", taskStatus, taskNames);
        Long dayNum = executionFlowsMapper.queryDaysNumber(taskStatus, taskNames);
        if (historyAvgData == null || historyAvgData.size() == 0) {
            for (int i = 0; i < 24; i++) {
                historyAvgList.add("0");
            }
        } else {
            for (Map.Entry<String, Object> entry : historyAvgData.entrySet()) {
                BigDecimal taskNum = new BigDecimal(String.valueOf(entry.getValue()));
                BigDecimal dayNumStr = new BigDecimal(dayNum);
                historyAvgList.add(String.valueOf(taskNum.divide(dayNumStr, 2, BigDecimal.ROUND_HALF_UP)));
            }
        }
        Map returnData = new HashMap();
        returnData.put("nowData", nowList);
        returnData.put("yesterdayData", yesterdayList);
        returnData.put("historyAvgData", historyAvgList);
        return JsonResult.ok(returnData);
    }

    /**
     * 获取 SLA
     *
     * @return
     * @author Bellamy
     * @since 2021-03-08
     */
    @Override
    public JsonResult getSlaInfo(String scheduleId, String projectId) throws Exception {
        DmpProjectSystemInfo publishToProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(Long.valueOf(projectId));

        //调用azkaban服务
        String azkabanApiUrl = publishToProjectSystemInfo.getAzkabanMonitorUrl();
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanApiUrl, redisTemplate);
        Map result = (Map) JSONObject.parse(azkabanApiUtils.getSlaInfo(scheduleId));
        SetSlaReq dto = new SetSlaReq();
        if (result != null && result.size() > 0) {
            List<Map> settings = (List<Map>) result.get("settings");
            List<String> slaEmails = (List<String>) JSONObject.parse(String.valueOf(result.get("slaEmails")));
            String emails = "";
            for (String email : slaEmails) {
                emails += "," + email;
            }
            dto.setSlaEmails(emails.substring(1));
            List<SetSlaRulesReq> list = new ArrayList<>();
            if (settings.size() > 0 && null != settings) {
                for (Map str : settings) {
                    SetSlaRulesReq slaRules = new SetSlaRulesReq();
                    String duration = String.valueOf(str.get("duration")).substring(0, String.valueOf(str.get("duration")).lastIndexOf("m"));
                    long m = Long.parseLong(duration);
                    long minute = (m) % 60;
                    long hour = (m / 60) % 24;
                    slaRules.setDuration(hour + ":" + minute);
                    slaRules.setFlow(String.valueOf(str.get("id")));
                    slaRules.setSlaRule(String.valueOf(str.get("rule")));
                    List<String> actions = (List<String>) JSONObject.parse(String.valueOf(str.get("actions")));
                    slaRules.setEmailNotification(false);
                    slaRules.setKill(false);
                    for (String action : actions) {
                        if ("EMAIL".equals(action)) {
                            slaRules.setEmailNotification(true);
                        } else if ("KILL".equals(action)) {
                            slaRules.setKill(true);
                        }
                    }
                    list.add(slaRules);
                }
            }
            dto.setSlaRule(list);
        }
        return JsonResult.ok(dto);
    }

    /**
     * 获取 flowId/任务名称
     *
     * @return
     * @author Bellamy
     * @since 2021-03-08
     */
    @Override
    public JsonResult getTaskFlowId(String projectId) throws Exception {
        List<Map> list = dmpNavigationTreeDao.queryTaskFlowId(projectId);
        if (list.size() > 0 && list != null) {
            for(Map map :list){
                map.put("name",CommonUtils.getAzkabanName(String.valueOf(map.get("taskId"))));
            }
        }
        return JsonResult.ok(list);
    }

}