package com.jz.dmp.modules.service.projconfig.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.StatuConstant;
import com.jz.common.persistence.BaseService;
import com.jz.dmp.modules.controller.projconfig.bean.DmpComputEngineParamBatch;
import com.jz.dmp.modules.controller.projconfig.bean.DmpComputEngineParamDto;
import com.jz.dmp.modules.controller.projconfig.bean.DmpComputEngineParamRequest;
import com.jz.dmp.modules.dao.projconfig.DmpComputEngineParamMapper;
import com.jz.dmp.modules.model.DmpComputEngineParam;
import com.jz.dmp.modules.service.projconfig.DmpComputEngineParamService;

/**
 * 计算引擎参数表服务的实现?
 * 
 * @author ybz
 *
 */
@Service("dmpComputEngineParamService")
public class DmpComputEngineParamServiceImpl extends BaseService implements DmpComputEngineParamService {

	private static Logger logger = LoggerFactory.getLogger(DmpComputEngineParamServiceImpl.class);

	@Autowired
	private DmpComputEngineParamMapper dmpComputEngineParamMapper;

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#findList(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineParamRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public PageInfoResponse<DmpComputEngineParamDto> findListWithPage(DmpComputEngineParamRequest dmpComputEngineParamRequest, HttpServletRequest httpRequest)
			throws Exception {
		PageInfoResponse<DmpComputEngineParamDto> pageInfoResponse = new PageInfoResponse<>();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpComputEngineParamRequest.getParamId() != null) {
			param.put("paramId", dmpComputEngineParamRequest.getParamId());
		}
		//引擎主键
		if (dmpComputEngineParamRequest.getEngineId() != null) {
			param.put("engineId", dmpComputEngineParamRequest.getEngineId());
		}
		//参数名称
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamName())) {
			param.put("paramName", dmpComputEngineParamRequest.getParamName());
		}
		//类型(0：引擎公共参数，1：引擎私配置参数)
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamType())) {
			param.put("paramType", dmpComputEngineParamRequest.getParamType());
		}
		//参数值
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamValue())) {
			param.put("paramValue", dmpComputEngineParamRequest.getParamValue());
		}
		//备注
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getRemark())) {
			param.put("remark", dmpComputEngineParamRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getDataStatus())) {
			param.put("dataStatus", dmpComputEngineParamRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpComputEngineParamRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpComputEngineParamRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpComputEngineParamRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpComputEngineParamRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpComputEngineParamRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpComputEngineParamRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpComputEngineParamRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpComputEngineParamRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpComputEngineParamRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpComputEngineParamRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpComputEngineParamRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpComputEngineParamRequest.getUpdateTimeEnd());
		}

		PageHelper.startPage(dmpComputEngineParamRequest.getPageNum(), dmpComputEngineParamRequest.getPageSize());
		List<DmpComputEngineParamDto> list = dmpComputEngineParamMapper.findList(param);
		PageInfo<DmpComputEngineParamDto> pageInfo = new PageInfo<>(list);

		pageInfoResponse.setCode(StatuConstant.SUCCESS_CODE);
		pageInfoResponse.setMessage("查询成功");
		pageInfoResponse.setData(pageInfo);
		return pageInfoResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#findList(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineParamRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpComputEngineParamDto> findList(DmpComputEngineParamRequest dmpComputEngineParamRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpComputEngineParamDto> baseBeanResponse = new BaseBeanResponse<>();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpComputEngineParamRequest.getParamId() != null) {
			param.put("paramId", dmpComputEngineParamRequest.getParamId());
		}
		//引擎主键
		if (dmpComputEngineParamRequest.getEngineId() != null) {
			param.put("engineId", dmpComputEngineParamRequest.getEngineId());
		}
		//参数名称
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamName())) {
			param.put("paramName", dmpComputEngineParamRequest.getParamName());
		}
		//类型(0：引擎公共参数，1：引擎私配置参数)
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamType())) {
			param.put("paramType", dmpComputEngineParamRequest.getParamType());
		}
		//参数值
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamValue())) {
			param.put("paramValue", dmpComputEngineParamRequest.getParamValue());
		}
		//备注
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getRemark())) {
			param.put("remark", dmpComputEngineParamRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getDataStatus())) {
			param.put("dataStatus", dmpComputEngineParamRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpComputEngineParamRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpComputEngineParamRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpComputEngineParamRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpComputEngineParamRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpComputEngineParamRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpComputEngineParamRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpComputEngineParamRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpComputEngineParamRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpComputEngineParamRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpComputEngineParamRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpComputEngineParamRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpComputEngineParamRequest.getUpdateTimeEnd());
		}

		List<DmpComputEngineParamDto> list = dmpComputEngineParamMapper.findList(param);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setDatas(list);
		return baseBeanResponse;
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#add(com.ycxc.vmts.entity.DmpComputEngineParam,
	 * javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpComputEngineParam> add(DmpComputEngineParam dmpComputEngineParam, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpComputEngineParam> baseBeanResponse = new BaseBeanResponse<>();

		/*$*setParamIdCodeReplace*$*/
		// 创建人
		dmpComputEngineParam.setCreateUserId(getHttpRequestUserId(httpRequest));
		// 创建时间
		dmpComputEngineParam.setCreateTime(new Date());

		dmpComputEngineParamMapper.insertSelective(dmpComputEngineParam);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("新增成功");
		baseBeanResponse.setData(dmpComputEngineParam);

		return baseBeanResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#edit(com.ycxc.vmts.entity.DmpComputEngineParam,
	 * javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpComputEngineParam> edit(DmpComputEngineParam dmpComputEngineParam, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpComputEngineParam> baseBeanResponse = new BaseBeanResponse<>();

		// 修改人
		dmpComputEngineParam.setUpdateUserId(getHttpRequestUserId(httpRequest));
		// 修改时间
		dmpComputEngineParam.setUpdateTime(new Date());

		dmpComputEngineParamMapper.updateByPrimaryKeySelective(dmpComputEngineParam);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("修改成功");
		baseBeanResponse.setData(dmpComputEngineParam);

		return baseBeanResponse;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#addOrEdit(com.ycxc.vmts.entity.DmpComputEngineParam,
	 * javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpComputEngineParam> addOrEdit(DmpComputEngineParam dmpComputEngineParam, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpComputEngineParam> baseBeanResponse = new BaseBeanResponse<>();
		
		Integer paramId = dmpComputEngineParam.getParamId();
		
		if (null==paramId) {//新增
			// 创建人
			dmpComputEngineParam.setCreateUserId(getHttpRequestUserId(httpRequest));
			// 创建时间
			dmpComputEngineParam.setCreateTime(new Date());

			dmpComputEngineParamMapper.insertSelective(dmpComputEngineParam);

			baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
			baseBeanResponse.setMessage("新增成功");
			baseBeanResponse.setData(dmpComputEngineParam);
		} else {//修改
			// 修改人
			dmpComputEngineParam.setUpdateUserId(getHttpRequestUserId(httpRequest));
			// 修改时间
			dmpComputEngineParam.setUpdateTime(new Date());

			dmpComputEngineParamMapper.updateByPrimaryKeySelective(dmpComputEngineParam);

			baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
			baseBeanResponse.setMessage("修改成功");
			baseBeanResponse.setData(dmpComputEngineParam);
		}
		
		return baseBeanResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#findById(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineParamRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpComputEngineParamDto> findById(Integer paramId, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpComputEngineParamDto> baseBeanResponse = new BaseBeanResponse<>();

		DmpComputEngineParamDto dmpComputEngineParamDto = dmpComputEngineParamMapper.findById(paramId);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setData(dmpComputEngineParamDto);

		return baseBeanResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineParamRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse deleteById(Integer paramId, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpComputEngineParamMapper.deleteByPrimaryKey(paramId);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineParamRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse softDeleteById(Integer paramId, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpComputEngineParamMapper.softDeleteByPrimaryKey(paramId);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineParamRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse delete(DmpComputEngineParamRequest dmpComputEngineParamRequest, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpComputEngineParamRequest.getParamId() != null) {
			param.put("paramId", dmpComputEngineParamRequest.getParamId());
		}
		//引擎主键
		if (dmpComputEngineParamRequest.getEngineId() != null) {
			param.put("engineId", dmpComputEngineParamRequest.getEngineId());
		}
		//参数名称
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamName())) {
			param.put("paramName", dmpComputEngineParamRequest.getParamName());
		}
		//类型(0：引擎公共参数，1：引擎私配置参数)
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamType())) {
			param.put("paramType", dmpComputEngineParamRequest.getParamType());
		}
		//参数值
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamValue())) {
			param.put("paramValue", dmpComputEngineParamRequest.getParamValue());
		}
		//备注
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getRemark())) {
			param.put("remark", dmpComputEngineParamRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getDataStatus())) {
			param.put("dataStatus", dmpComputEngineParamRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpComputEngineParamRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpComputEngineParamRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpComputEngineParamRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpComputEngineParamRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpComputEngineParamRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpComputEngineParamRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpComputEngineParamRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpComputEngineParamRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpComputEngineParamRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpComputEngineParamRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpComputEngineParamRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpComputEngineParamRequest.getUpdateTimeEnd());
		}

		dmpComputEngineParamMapper.delete(param);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineParamRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse softDelete(DmpComputEngineParamRequest dmpComputEngineParamRequest, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpComputEngineParamRequest.getParamId() != null) {
			param.put("paramId", dmpComputEngineParamRequest.getParamId());
		}
		//引擎主键
		if (dmpComputEngineParamRequest.getEngineId() != null) {
			param.put("engineId", dmpComputEngineParamRequest.getEngineId());
		}
		//参数名称
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamName())) {
			param.put("paramName", dmpComputEngineParamRequest.getParamName());
		}
		//类型(0：引擎公共参数，1：引擎私配置参数)
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamType())) {
			param.put("paramType", dmpComputEngineParamRequest.getParamType());
		}
		//参数值
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getParamValue())) {
			param.put("paramValue", dmpComputEngineParamRequest.getParamValue());
		}
		//备注
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getRemark())) {
			param.put("remark", dmpComputEngineParamRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpComputEngineParamRequest.getDataStatus())) {
			param.put("dataStatus", dmpComputEngineParamRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpComputEngineParamRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpComputEngineParamRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpComputEngineParamRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpComputEngineParamRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpComputEngineParamRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpComputEngineParamRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpComputEngineParamRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpComputEngineParamRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpComputEngineParamRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpComputEngineParamRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpComputEngineParamRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpComputEngineParamRequest.getUpdateTimeEnd());
		}

		dmpComputEngineParamMapper.softDelete(param);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineParamService#addBatch(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineParamVo, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpComputEngineParamBatch> addBatch(DmpComputEngineParamBatch dmpComputEngineParamBatch, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpComputEngineParamBatch> baseBeanResponse = new BaseBeanResponse<>();

		List<DmpComputEngineParam> dmpComputEngineParams = dmpComputEngineParamBatch.getDmpComputEngineParams();

		if (CollectionUtils.isEmpty(dmpComputEngineParams)) {
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			baseBeanResponse.setMessage("数据不能为空！");
			return baseBeanResponse;
		}

		dmpComputEngineParams = dmpComputEngineParams.stream().map(dmpComputEngineParam -> {
			/*$*setParamIdCodeReplace*$*/
			// 创建人
			try {
				dmpComputEngineParam.setCreateUserId(getHttpRequestUserId(httpRequest));
			} catch (Exception e) {
				logger.info(e.getMessage());
				e.printStackTrace();
			}
			// 创建时间
			dmpComputEngineParam.setCreateTime(new Date());

			return dmpComputEngineParam;
		}).collect(Collectors.toList());

		dmpComputEngineParamMapper.insertBatch(dmpComputEngineParams);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("批量新增成功");
		baseBeanResponse.setData(dmpComputEngineParamBatch);

		return baseBeanResponse;
	}

	/**
	 *批量删除
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse deleteByIds(List<Integer> idList, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpComputEngineParamMapper.deleteByIds(idList);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");

		return baseResponse;
	}

	/**
	 *批量软删除
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse softDeleteByIds(List<Integer> idList, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpComputEngineParamMapper.softDeleteByIds(idList);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");

		return baseResponse;
	}

}
